import React from 'react';
import { findDOMNode } from 'react-dom';
import moment from 'moment';
import assign from 'object-assign';
import { Fields } from 'FieldTypes';

// import {
// 	components,
// } from 'kadm-component';
var {
	LaddaBtn,
	Modal,
} = require('../utilsForReactDom');


import FormHeading from './FormHeading';
import AltText from './AltText';
import FooterBar from './FooterBar';
import InvalidFieldType from '../../shared/InvalidFieldType';

//import { itemDeleteItem } from 'keystone/admin/client/App/screens/Item/actions';

import { upcase } from '../../../utils/string';
//import Form from "antd/lib/form";
//import Input from "antd/lib/input";
//import Row from "antd/lib/row";
//import Col from "antd/lib/col";
//import Button from "antd/lib/button";



import {
	message,
	Group,
	Form,
	Input,
	Row,
	Col,
	Button,
} from 'antdlib'

function getNameFromData (data) {
	if (typeof data === 'object') {
		if (typeof data.first === 'string' && typeof data.last === 'string') {
			return data.first + ' ' + data.last;
		} else if (data.id) {
			return data.id;
		}
	}
	return data;
}

var EditForm = React.createClass({
	displayName: 'EditForm',
	propTypes: {
		data: React.PropTypes.object,
		list: React.PropTypes.object,
	},
	contextTypes:{
		router:React.PropTypes.object.isRequired,
	},
	getInitialState () {
		return {
			values: assign({}, this.props.data.fields),
			confirmationDialog: null,
			loading: false,
			// lastValues: null, // used for resetting
			modalDisable: true,
			layout:"horizontal",
		};
	},
	getFieldProps (field) {
		const props = assign({}, field);
		const alerts = this.state.alerts;
		// Display validation errors inline
		if (alerts && alerts.error && alerts.error.error === 'validation errors') {
			if (alerts.error.detail[field.path]) {
				// NOTE: This won't work yet, as ElementalUI doesn't allow
				// passed in isValid, only invalidates via internal state.
				// PR to fix that: https://github.com/elementalui/elemental/pull/149
				props.isValid = false;
			}
		}
		props.layout = this.state.layout;
		props.formItemLayout = props.layout == "horizontal" ? {
			labelCol: { span: 4 },
			wrapperCol: { span: 14 },
		} : null;
		props.form = this.props.form;
		props.value = this.state.values[field.path];
		props.values = this.state.values;
		props.onChange = this.handleChange;
		props.mode = 'edit';
		return props;
	},
	handleChange (event) {
		const values = assign({}, this.state.values);

		values[event.path] = event.value;
		this.setState({ values });
	},
	confirmReset (event) {

		this.refs.Modal.show({
			title:"提示",
			body:<span>是否重置名为<strong>${this.props.data.name}</strong>的对象?</span>,
			leftItems:[
				<Button type="danger" size="large" onClick={this.handleReset.bind(this)}>重置</Button>
			]

		});
		event.preventDefault();
	},
	handleReset () {
		this.props.form.resetFields();
		this.refs.Modal.hide();
	},
	confirmDelete () {

		this.refs.Modal.show({
			title:"提示",
			body:(<span>是否删除当前对象:<strong>${this.props.data.name}?</strong></span>),
			leftItems:[
				<Button className="pull-right" onClick={this.handleDelete.bind(this)} loading={this.state.loading} type="danger" size="large">
					删除
				</Button>
			]

		});
	},
	handleDelete () {
		const { data } = this.props;
		let list = this.props.list;
		let self = this;
		this.setState({
			loading:true,
		})
		list.deleteItem(data.id, (err, data) => {
			// TODO Proper error handling
			this.setState({
				loading:false,
			})
			if (err) {
				message.error(err.error);
			} else {

				self.props.router.goBack();
				self.refs.Modal.hide();
			}

		});
	},
	handleKeyFocus () {
		const input = findDOMNode(this.refs.keyOrIdInput);
		input.select();
	},
	updateItem (btn) {
		const { data, list } = this.props;
		this.props.form.validateFields((err, values) => {
			if (!err) {
				//console.log('Received values of form: ', values);
				let formData = new FormData();
				_.forEach(values, (value, key) => {
					if (Array.isArray(value) && value.length == 0){
						//远择下拉类，如果子元素为0，表格不传值
						return;
					}
					else if (typeof value == 'object' && value != null && (value._isAMomentObject || value._JSONObject)){
						//时间类元母，执行field的format
						value = value.format();
					} else if (typeof value == 'object' && value != null) {
						//const fieldType = Fields[this.props.list.fields[key].type];
						//console.log(fieldType)
						value = this.refs['field_ref_' + key].format();
					}

					formData.append(key, value)
				});
				btn.start();
				list.updateItem(data.id, formData, (err, data) => {
					// TODO: implement smooth scolling
					scrollTo(0, 0); // Scroll to top
					btn.stop();
					if (err) {
						message.error(err.error)

					} else {
						// Success, display success flash messages, replace values
						// TODO: Update key value
						message.success('成功保存');
						this.context.router.goBack();
					}
				});
			}
			else {
				btn.stop();
				console.log(err);
			}
		});

	},
	renderKeyOrId () {
		var className = 'EditForm__key-or-id';
		var list = this.props.list;

		if (list.nameField && list.autokey && this.props.data[list.autokey.path]) {
			return (
				<div className={className}>
					<AltText
						component="span"
						modifiedLabel="ID:"
						modifiedValue={null}
						normalLabel={`${upcase(list.autokey.path)}: `}
						normalValue={null}
						title="Press <alt> to reveal the ID"
						className="EditForm__key-or-id__label" />
					<AltText
						component="span"
						modifiedLabel=""
						modifiedValue={<input ref="keyOrIdInput" onFocus={this.handleKeyFocus} value={this.props.data.id} className="EditForm__key-or-id__input" readOnly />}
						normalLabel={null}
						normalValue={<input ref="keyOrIdInput" onFocus={this.handleKeyFocus} value={this.props.data[list.autokey.path]} className="EditForm__key-or-id__input" readOnly />}
						title="Press <alt> to reveal the ID"
						className="EditForm__key-or-id__field" />
				</div>
			);
		} else if (list.autokey && this.props.data[list.autokey.path]) {
			return (
				<div className={className}>
					<span className="EditForm__key-or-id__label">{list.autokey.path}: </span>
					<div className="EditForm__key-or-id__field">
						<input ref="keyOrIdInput" onFocus={this.handleKeyFocus} value={this.props.data[list.autokey.path]} className="EditForm__key-or-id__input" readOnly />
					</div>
				</div>
			);
		} else if (list.nameField) {
			return (
				<div className={className}>
					<span className="EditForm__key-or-id__label">ID: </span>
					<div className="EditForm__key-or-id__field">
						<input ref="keyOrIdInput" onFocus={this.handleKeyFocus} value={this.props.data.id} className="EditForm__key-or-id__input" readOnly />
					</div>
				</div>
			);
		}
	},
	renderNameField () {
		var nameField = this.props.list.nameField;
		var nameIsEditable = this.props.list.nameIsEditable;
		var wrapNameField = field => (
			<div className="EditForm__name-field">
				{field}
			</div>
		);
		if (nameIsEditable) {
			var nameFieldProps = this.getFieldProps(nameField);
			nameFieldProps.label = null;
			nameFieldProps.size = 'full';
			nameFieldProps.inputProps = {
				className: 'item-name-field',
				placeholder: nameField.label,
				size: 'lg',
			};
			return wrapNameField(
				React.createElement(Fields[nameField.type], nameFieldProps)
			);
		} else {
			return wrapNameField(
				<h2>{this.props.data.name || '(no name)'}</h2>
			);
		}
	},
	renderFormElements () {
		var headings = 0;

		let hiddenEle = _.filter(this.props.list.fields, (o) => {
			return o.hidden;
		});

		let {getFieldDecorator} = this.props.form;

		hiddenEle = _.map(hiddenEle, (o) => {
			headings++;
			
			var props = this.getFieldProps(o);

			return getFieldDecorator(
				props.path,
				{
					rules: [],
					initialValue:props.value
				}
			)(
				<Input key={'h' + headings} name={props.path} type='hidden' /> 
			);
		})
		let formEle = this.props.list.uiElements.map((el) => {
			if (el.type === 'heading') {
				headings++;
				el.options.values = this.state.values;
				el.key = 'h-' + headings;
				return React.createElement(FormHeading, el);
			}

			if (el.type === 'field') {
				var field = this.props.list.fields[el.field];
				var props = this.getFieldProps(field);
				if (typeof Fields[field.type] !== 'function') {
					return React.createElement(InvalidFieldType, { type: field.type, path: field.path, key: field.path });
				}
				if (props.dependsOn) {
					props.currentDependencies = {};
					Object.keys(props.dependsOn).forEach(dep => {
						props.currentDependencies[dep] = this.state.values[dep];
					});
				}
				props.key = field.path;
				props.ref = 'field_ref_' + field.path;

				return React.createElement(Fields[field.type], props);
			}
		}, this);

		return hiddenEle.concat(formEle);
	},
	renderFooterBar () {
		//old foot bar
		let isRoot = false;
		if (KAdm.roles.indexOf('root') != -1){
			isRoot = true;
		}

		let key = this.props.list.key;
		let id = this.props.data.id;

		let availSave = false;
		let availDel = false;
		if (key){
			if (KAdm.idAuthModel.indexOf(key) != -1){
				key = id + '@' + key;
			}
			if (KAdm.roles.indexOf('root') != -1 || KAdm.isCreator || KAdm.idAuthModel.indexOf(this.props.list.key) > -1){
				availDel = availSave = true;
			} else {
				if (KAdm.permissions[key].indexOf('add') != -1)
					availSave = true;
				if (KAdm.permissions[key].indexOf('delete') != -1)
					availDel = true;
			}
			
		}


		availSave = isRoot || availSave;
		availDel = isRoot || availDel;

		var buttons = [
			<LaddaBtn disabled={!availSave} ref="saveBtn" type="primary" clickHandler={this.updateItem} >
				保存
			</LaddaBtn>

		];
		buttons.push(
			<Button key="reset" type="dashed" size='large' onClick={this.confirmReset.bind(this)}>
				重置修改
			</Button>
		);
		if (!this.props.list.nodelete) {
			buttons.push(
				<Button disabled={!availDel} type="danger" key="del" size='large' onClick={this.confirmDelete.bind(this)}>
					删除
				</Button>
			);
		}
		return (
			<Button.Group>
				{buttons}
			</Button.Group>
		);
	},
	renderTrackingMeta () {
		return null;
		if (!this.props.list.tracking) return null;

		var elements = [];
		var data = {};

		if (this.props.list.tracking.createdAt) {
			data.createdAt = this.props.data.fields[this.props.list.tracking.createdAt];
			if (data.createdAt) {
				elements.push(
					<Form.Item key="createdAt" label="Created on" labelCol={{span:4}} wrapperCol={{span:20}}>
						<Input diabled title={moment(data.createdAt).format('DD/MM/YYYY HH:mm:ss')}>{moment(data.createdAt).format('Do MMM YYYY')}</Input>
					</Form.Item>
				);
			}
		}

		if (this.props.list.tracking.createdBy) {
			data.createdBy = this.props.data.fields[this.props.list.tracking.createdBy];
			if (data.createdBy && data.createdBy.name) {
				let createdByName = getNameFromData(data.createdBy.name);
				if (createdByName) {
					elements.push(
						<Form.Item key="createdBy" label="Created by" labelCol={{span:4}} wrapperCol={{span:20}}>
							<Input diabled>{data.createdBy.name.first} {data.createdBy.name.last}</Input>
						</Form.Item>
					);
				}
			}
		}

		if (this.props.list.tracking.updatedAt) {
			data.updatedAt = this.props.data.fields[this.props.list.tracking.updatedAt];
			if (data.updatedAt && (!data.createdAt || data.createdAt !== data.updatedAt)) {
				elements.push(
					<Form.item key="updatedAt" label="Updated on" labelCol={{span:4}} wrapperCol={{span:20}}>
						<Input diabled title={moment(data.updatedAt).format('DD/MM/YYYY HH:mm:ss')}>{moment(data.updatedAt).format('Do MMM YYYY')}</Input>
					</Form.item>
				);
			}
		}

		if (this.props.list.tracking.updatedBy) {
			data.updatedBy = this.props.data.fields[this.props.list.tracking.updatedBy];
			if (data.updatedBy && data.updatedBy.name) {
				let updatedByName = getNameFromData(data.updatedBy.name);
				if (updatedByName) {
					elements.push(
						<Form.Item key="updatedBy" label="Updated by" labelCol={{span:4}} wrapperCol={{span:20}}>
							<Input diabled>{data.updatedBy.name.first} {data.updatedBy.name.last}</Input>
						</Form.Item>
					);
				}
			}
		}

		return Object.keys(elements).length ? (
			<div className="EditForm__meta">
				<h3 className="form-heading">Meta</h3>
				{elements}
			</div>
		) : null;
	},
	render () {
		return (
			<Form ref="editForm"  className="EditForm-container" type={this.state.layout} layout={this.state.layout}>
				<Row>
					<Col span={22}>
							{this.renderNameField()}
							{this.renderKeyOrId()}
							{this.renderFormElements()}
							{this.renderTrackingMeta()}

					</Col>
					<Col span={2}><span /></Col>
				</Row>
				{this.renderFooterBar()}
				<Modal ref="Modal"></Modal>

			</Form>
		);
	},
});

module.exports = Form.create({
	onFieldsChange:(props, fields)=>{
		console.log(fields);
	},
	onValuesChange:(props, values)=>{
		console.log(values);
	}
})(EditForm);
