export const ITEM_LOAD_DATAS = "ITEM_LOAD_DATAS";
export function loadItemsData (options = {}) {
	return (dispatch, getState) => {
		dispatch({ type: ITEM_LOAD_DATAS });
		const state = getState();
		const currentList = state.lists.currentModel;
		currentList.loadItems({
			search: state.active.search,
			filters: state.active.filters,
			sort: state.active.sort,
			columns: state.active.columns,
			page: state.lists.page,
		}, (err, items) => {
			if (items) {
				// if (page.index !== drag.page && drag.item) {
				// 	// add the dragging item
				// 	if (page.index > drag.page) {
				// 		_items.results.unshift(drag.item);
				// 	} else {
				// 		_items.results.push(drag.item);
				// 	}
				// }
				// _itemsResultsClone = items.results.slice(0);
				//

				// TODO Reenable this
				// if (options.success && options.id) {
				// 	// flashes a success background on the row
				// 	_rowAlert.success = options.id;
				// }
				// if (options.fail && options.id) {
				// 	// flashes a failure background on the row
				// 	_rowAlert.fail = options.id;
				// }
				dispatch(itemsLoaded(items));
			} else {
				dispatch(itemLoadingError(err));
			}
		});
	};
}

/**
 * Select an item
 *
 * @param  {String} itemId The item ID
 */
export const ITEM_SELECT = "ITEM_SELECT";
export const ITEM_LOAD_DATA = "ITEM_LOAD_DATA";
export function loadItemData (itemId) {
	return (dispatch, getState) => {
		dispatch({
			type: ITEM_SELECT,
			id:itemId,
		});
		dispatch({
			type: ITEM_LOAD_DATA,
		});
		const state = getState();
		const model = state.lists.currentModel;
		// Load a specific item with the utils/List.js helper
		model.loadItem(itemId, { drilldown: true }, (err, itemData) => {
			if (err || !itemData) {
				console.log('Error loading item data', err);
				dispatch(itemDataLoadingError(err));
			} else {
				dispatch(itemDataLoaded(itemData));
			}
		});
	};
}
/**
 * Called when data of the current item is loaded
 *
 * @param  {Object} data The item data
 */
export const ITEM_DATA_LOADING_SUCCESS = "ITEM_DATA_LOADING_SUCCESS";
export function itemDataLoaded (data) {
	return {
		type: ITEM_DATA_LOADING_SUCCESS,
		data,
	};
}

/**
 * Called when there was an error during the loading of the current item data,
 * will retry loading the data ever NETWORK_ERROR_RETRY_DELAY milliseconds
 *
 * @param  {Object} error The error
 */
export const ITEM_PROCESS_ERROR = "ITEM_PROCESS_ERROR"
export function itemDataLoadingError (err) {
	return {
		type: ITEM_PROCESS_ERROR,
		error: err,
	};
}

/**
 * Deletes an item and optionally redirects to the current list URL
 *
 * @param  {String} id     The ID of the item we want to delete
 * @param  {Object} router A react-router router object. If this is passed, we
 *                         redirect to Keystone.adminPath/currentList.path!
 */
export const ITEM_DELETE= "ITEM_DELETE";
export function itemDeleteItem (id, router) {
	return (dispatch, getState) => {
		const list = getState().lists.currentModel;
		list.deleteItem(id, (err) => {
			// If a router is passed, redirect to the current list path,
			// otherwise stay where we are
			if (router) {
				router.push(`${Keystone.adminPath}/${list.path}`);
			}
			// TODO Proper error handling
			if (err) {
				alert('Error deleting item, please try again!');
				console.log(err);
			} else {
				dispatch({
					type: ITEM_DELETE,
					id,
				});
			}
		});
	};
}

export const ITEM_CREATE= "ITEM_CREATE";
export function itemCreate (formData, callback) {
	return (dispatch, getState) => {
		const list = getState().lists.currentModel;
		list.createItem(formData,(err, data) => {
			if (data) {
				dispatch(itemCreateSuccess(data));
			} else {
				if (err.error === 'database error') {
					err.error = err.detail.errmsg;
				}
				dispatch(itemCreateFail(err));
			}
			callback(err, data);
		});
	};
}

export const ITEM_CREATE_SUCCESS= "ITEM_CREATE_SUCCESS";
export function itemCreateSuccess (data) {
	return {
		type:ITEM_CREATE_SUCCESS,
		createdData:data
	}
}
export const ITEM_CREATE_FAIL= "ITEM_CREATE_FAIL";
export function itemCreateFail (err) {
	return {
		type:ITEM_CREATE_FAIL,
		err:err
	}
}
