/**
 * Created by kinfen on 16/7/26.
 */

import assign from "object-assign";
import _ from "lodash";
import {
	setActiveList,
	setActiveSort,
} from "./active/actions";


let findModelName = (name) => {
	if (KAdm.lists.hasOwnProperty(name))
		//return directly if supply a field name
		return name;
	else{
		//find field name to return if supplied a path name;
		let targetName = _.findKey(KAdm.lists, (o) => {
			return o.path == name;
		});
		return targetName;
	}
}

export const LIST_SELECT_MODEL = "LIST_SELECT_MODEL";
export function selectModel (modelName) {
	modelName = findModelName(modelName);

	return (dispatch, getState) => {
		let lists = getState().lists;
		let actions = KAdm.menuActions;
		let permissions = KAdm.permissions[modelName];
		if (permissions) actions = _.filter(actions, (o) => permissions.indexOf(o.code) != -1);
		else actions = [];
		dispatch ({
			type:LIST_SELECT_MODEL,
			currentModel:lists.model[modelName],
			menuActions:actions,
		});
		dispatch(setActiveList(lists.model[modelName], modelName));
	};
}


// export const LIST_SELECT_LIST = "LIST_SELECT_LIST";
// export function selectList(list){
// 	list = findModelName(list);
// 	return (dispatch, getState) => {
// 		let lists = getState().lists;

// 		let actions = KAdm.menuActions;
// 		let permissions = KAdm.permissions[list];
// 		if (permissions) actions = _.filter(actions, (o) => permissions.indexOf(o.code) != -1);
// 		else actions = [];

// 		dispatch ({
// 			type:LIST_SELECT_LIST,
// 			list:lists.model[list],
// 			menuActions:actions,
// 		});
// 		dispatch(setActiveList(lists.model[list], list));
// 		if (permissions) {
// 			dispatch(loadDataFromState());
// 		}
// 	};
// }


export const LIST_PAGE = "LIST_PAGE";
export function listPage(page){
	return (dispatch, getState) => {
		dispatch({
			type: LIST_PAGE,
			page,
		})
		dispatch(loadDataFromState());
	}
}

export const LIST_CONTENT_LOADING = "LIST_CONTENT_LOADING";
export function contentLoading (isLoading) {
	return {
		type: LIST_CONTENT_LOADING,
		loading:isLoading,
	};
}


/**
 * 更新content内容
 */
export const LIST_CONTENT_LOAD = "LIST_CONTENT_LOAD";
// function tableList(data) {
// 	var self = this;
// 	data = _.map(data, function (obj) {
// 		obj.fields.id = obj.id;
// 		return _.mapValues(obj.fields, function (target) {
// 			if (typeof(target) == "object" && target.name) {
// 				return target.name;
// 			}
// 			return target;
// 		})

// 	});
// 	return data;
// }


function contentRecieve(data){
	return (dispatch, getState) => {

		dispatch({
			type:LIST_CONTENT_LOAD,
			totalPages:Math.ceil(data.count / getState().lists.page.size),
			total:data.count,
			data:data.results,
		});
	};
}

export const LIST_CLEAR_CONTENT = "LIST_CLEAR_CONTENT";
export function clearContent(){
	return (dispatch, getState) => {

		dispatch({
			type:LIST_CLEAR_CONTENT,
			totalPages:0,
			total:0,
			data:[]
		});
	};
}

function contentError(error){
	return {
		type:LIST_CONTENT_ERROR,
		error
	}
}

/**
 *
 *
 * @returns {function(*, *)}
 *
 * search: "",
 * filters:[{
 * 	field:{}
 * 	value:{ inverse:false, value:[]}
 * }],
 * sort:{
 * 	input:"-publishedDate"
 * 	isDefaultSore:true,
 * 	rawInput:"-publishedDate",
 * 	paths:[{field}]
 * 	},
 * 	column:[{field:{}, label:"", path:""}],
 * 	page:{
 * 		index:1,
 * 		size:1
 *
 * 	}
 *
 */

export function loadDataFromState(){

	return (dispatch, getState) => {
		dispatch(contentLoading(true));
		const state = getState();
		const currentList = state.lists.currentModel;
		currentList.loadItems({
			search: state.active.search,
			filters: state.active.filters,
			sort: state.active.sort,
			columns: state.active.columns,
			page: state.lists.page,
		}, (err, items) => {
			if (items) {
				dispatch(contentRecieve(items));
			} else {
				dispatch(contentError(err));
			}
			dispatch(contentLoading(false));
		});
	}

}

export const LIST_POPOVER_LOCK_CHANGE = "LIST_POPOVER_LOCK_CHANGE";
export function lockPopover(popoverLock){
	return (dispatch, getState) => {
		dispatch({
			type:LIST_POPOVER_LOCK_CHANGE,
			popoverLock,
		});
	};
}



export const LIST_CONTENT_ERROR = "LIST_CONTENT_ERROR";



