/**
 * Created by kinfen on 16/8/19.
 */

import _ from 'lodash';
import assign from 'object-assign';

import {
	LIST_SET_ACTIVE_LIST,
	LIST_ADD_FILTER,
	LIST_CLEAR_ALL_FILTERS,
	LIST_CLEAR_FILTER,
	LIST_SET_ACTIVE_COLUMNS,
	LIST_SET_ACTIVE_SEARCH,
	LIST_SET_ACTIVE_SORT,
} from "./actions";

const initialState = {
	columns: null,
	filters: [],
	search: '',
	sort: {
		input: '',
		isDefaultSort: false,
		paths: [],
		rawInput: '',
	},
};

/**
 * Manage the active state
 */
function active (state = initialState, action) {
	switch (action.type) {
		case LIST_SET_ACTIVE_LIST:
			let expandColumns = action.list.expandColumns(action.list.defaultColumns);
			return assign({}, state, {
				id: action.id,
				columns: state.id == action.id ? state.columns || expandColumns : expandColumns,
				filters: [],
				search: '',
				sort: action.list.expandSort(action.list.defaultSort),
			});
		case LIST_SET_ACTIVE_SEARCH:
			return assign({}, state, {
				search: action.searchString,
			});
		case LIST_SET_ACTIVE_SORT:
			return assign({}, state, {
				sort: action.sort,
			});
		case LIST_SET_ACTIVE_COLUMNS:
			return assign({}, state, {
				columns: action.columns,
			});
		case LIST_ADD_FILTER:
			return assign({}, state, {
				// Override existing filter with field path,
				// otherwise add to filters array
				filters: _.unionWith([action.filter], state.filters, (stateFilter, actionFilter) => {
					return stateFilter.field.path === actionFilter.field.path;
				}),
			});
		case LIST_CLEAR_FILTER:
			let newFilters = _.filter(state.filters, (filter) => {
				return filter.field.path !== action.path;
			});
			return assign({}, state, {
				filters: newFilters,
			});
		case LIST_CLEAR_ALL_FILTERS:
			return assign({}, state, {
				filters: [],
			});
		default:
			return state;
	}
}

export default active;
