/**
 * Created by kinfen on 16/2/1.
 */
'use strict';
import React from 'react';
import { connect } from 'react-redux';
import { Link } from 'react-router';
import _ from 'lodash';
import path from "path"
//
// import {BootstrapTable, TableHeaderColumn} from 'react-bootstrap-table';
//

import {
	Table,
	Button,
	Card,
	Popover,
	Tooltip,
	message,
} from 'antdlib';

import Component from '../KComponent';
import reduxManager from 'kadm-redux-mng';
import control from '../../shared/Control';
import {
	contentLoading,
	loadDataFromState,
	listPage,
	clearContent,
	
} from './actions';
import {
	setActiveSort,
	setFilter,
} from "./active/actions";



import {Columns} from 'FieldTypes';

import ListToolBar from "./components/ListToolBar"
import ListFilter from './components/ListFilters';
import {
	Pagination,
	Modal,
	AlertBox,
	LaddaBtn,
} from '../utilsForReactDom'

import ItemCreateModal from "./components/ItemCreateModal";
import RoleAppointModal from "./components/RoleAppointModal";
import ModuleAppointModal from "./components/ModuleAppointModal";
import ModelAppointModal from "./components/ModelAppointModal";



const Actions = {
	add:"add",
	delete:"delete",
	setrole:"setrole",
	setmodule:"setmodule",
	setmodel:"setmodel",
}


class ListView extends Component{
	constructor(props){
		super(props);
		this.sector  = "#table";
		this.sortName = null;
		this.sortOrder = null;
		this.updateCount = 0;

		this.state = {
			selectedRowKeys:[],
			actions:KAdm.menuActions,
		}
	}

	componentWillReceiveProps(nextProps){
		if (this.props.currentModel && nextProps.currentModel.key != this.props.currentModel.key){
			this.props.store.dispatch(clearContent());
			if (this.props.initFilter){
				// when change model and there is initFilter, keep inapps filters
				this.props.store.dispatch(setFilter(this.props.initFilter.filter, {
					inverted:false,
					value:[this.props.initFilter.value],
				}))
			}
			this.props.store.dispatch(loadDataFromState());
			
		}
		if (!_.isEqual(nextProps.initFilter, this.props.initFilter)){
			//in any case when initFilter change, set new  filters
			// this.props.store.dispatch(loadDataFromState());
			this.props.store.dispatch(clearContent());
			this.props.store.dispatch(setFilter(nextProps.initFilter.filter, {
				inverted:false,
				value:[nextProps.initFilter.value],
			}))
		}
		
	}
	fields(columns){
		var self = this;

		let sort = this.props.sort;
		var modelObj = this.props.currentModel;

		let fieldsList = _.map(columns, (column) => {
			const renderFunc = (text, row, index) => {
				var ColumnType = Columns[column.type] || Columns.__unrecognised__;
				var url = path.join(
					this.props.itemUrlPrefix ? this.props.itemUrlPrefix : this.context.adminPath,
					"item",
					modelObj.key,
					row.id
				)
				var linkTo = column.field && column.field.path == "name" ? url : null;

				if (ColumnType)
					return <ColumnType key={column.path} list={modelObj} col={column} data={row} linkTo={linkTo} />;
				else
					return <Link to={url}>{row.fields[column.type]}</Link>
			}
			let item = _.extend(column, {
					align: column.path == "name" ? "left" : "center",
					hidden: column.path == "id" ? true : false,
					render:renderFunc,
					title:column.label,
					dataIndex:column.path,
					sorter:true,
					sortOrder:column.path == sort.paths[0].path ? (sort.paths[0].invert ? "descend" : "ascend") : null,
				}
			);
			return column;
		});

		//check for controls
		if (modelObj.controls){
			let column = {
				type:'control',
				path:'control',
				label:'控制',
				sorter:false,
				width:100,
			};
			const renderFunc = (text, row, index) => {
				var ColumnType = Columns[column.type]
				return <ColumnType key="control" list={modelObj} col={column} data={row} controls={this.props.currentModel.controls} />;
			}
			let item = _.extend(column, {
					align: column.path == "name" ? "left" : "center",
					hidden: column.path == "id" ? true : false,
					render:renderFunc,
					title:column.label,
					dataIndex:column.path,
					sorter:false
				}
			);
			fieldsList.push(item);
		}
		return fieldsList;
	}

	createItemHandler(e)
	{
		this.refs.createModal.show({
			created:false,
		});
	}
	setRoleHandler(e){

		if (this.props.currentModel.key != 'User'){
			return message.error('只能对User使用Role授权')
		}

		let ids = this.state.selectedRowKeys;
		this.refs.roleappoint.show({
			ids,
		});
	}
	setModuleHandler(e){

		if (this.props.currentModel.key != 'Role'){
			return message.error('只能对Role使用Module授权')
		}

		let ids = this.state.selectedRowKeys;
		this.refs.moduleappoint.show({
			ids,
		});
	}
	setModelHandler(e){
		if (this.props.currentModel.key != 'Role'){
			return message.error('只能对Role使用Model授权')
		}
		let ids = this.state.selectedRowKeys;
		this.refs.modelappoint.show({
			ids,
		});
	}
	removeItemHandler (e)
	{

		// if ($(e.currentTarget).hasClass("disabled")) return;

		var ids = this.state.selectedRowKeys;
		var data = this.props.data;
		var selections = _.filter(data, function(o) { return ids.indexOf(o.id) != -1 });
		if (this.state.selectedRowKeys.length == 0)
		{
			this.refs.Modal.show({
				title:"提示",
				body:"请勾选需要删除的栏目"
			});
			return;
		}
		var list = [];
		for (var i = 0; i < selections.length; i++)
		{
			var node = selections[i];
			list.push(<li key={i}>{node.name}</li>);
		}
		var bodyElement = (
			<div>
				<div>是否确认删除以下{list.length}个对象</div>
				<ul>
					{list}
				</ul>
			</div>
		);

		this.refs.Modal.show({
			title:"提示",
			body:bodyElement,
			leftItems:[
				<LaddaBtn  type="danger" size='large'  clickHandler={this.removeItem.bind(this)}>删除</LaddaBtn>
			],
			rightItems:[{
				name:"关闭",
				style:"default",
				data:"dismiss"
			}]

		});
	}
	removeItem(btn){
		var self = this;
		var ids = this.state.selectedRowKeys;
		btn.start();
		var url = path.join(this.context.adminPath, this.props.apiPath, this.props.currentModel.key);
		control.api({
			url:url,
			type:"POST",
			data:{
				action:"delete",
				ids:ids
			},
			success:function(data)
			{
				btn.stop()
				if (data.status===1)
				{
					self.props.store.dispatch(loadDataFromState());
				}
				else {
					self.props.store.dispatch(contentLoading(false));
				}
				self.refs.Modal.hide();
			},
			error:function(){
				self.props.store.dispatch(contentLoading(false));
				btn.stop();
			}
		}, true);
	}
	componentWillMount(){
		
		// this.props.store.dispatch(loadDataFromState());
		if (this.props.initFilter){
			this.props.store.dispatch(setFilter(this.props.initFilter.filter, {
				inverted:false,
				value:[this.props.initFilter.value],
			}))
		} else {
			this.props.store.dispatch(clearContent());
			this.props.store.dispatch(loadDataFromState());
		}
	}
	// shouldComponentUpdate(){
	// 	return this.props.currentModel;
	// }
	componentWillUpdate(){

	}
	componentDidUpdate(){

	}
	toggleDeleteBtnState(){
		var rowsKey = this.state.selectedRowKeys;
		if (rowsKey.length > 0){
			$("#table-toolbar .btn.delete").removeClass("disabled")
		}
		else {
			$("#table-toolbar .btn.delete").addClass("disabled")
		}
	}
	onSelectionChange(selectedRowKeys){
		this.setState({ selectedRowKeys });
		this.toggleDeleteBtnState();
	}


	componentDidMount () {

	}

	actionsMapMethod(action){

		switch (action){
			case Actions.add:
				return  this.createItemHandler.bind(this);
				break;
			case Actions.delete:
				return this.removeItemHandler.bind(this);
				break;
			case Actions.setrole:
				return this.setRoleHandler.bind(this);

				break;
			case Actions.setmodule:
				return this.setModuleHandler.bind(this);

				break;
			case Actions.setmodel:
				return this.setModelHandler.bind(this);

				break;
		}

	}
	renderActions(){

		if (!this.props.menuActions){
			return null
		}
		let menuActions = this.props.menuActions;
		// [{
		// 	icon:'add',
		// 	label:'setmodule',
		// 	code:'setmodule',
		// }]
		let actionList = _.filter(menuActions, (o) => o.code != 'read');
		let actions = _.map(actionList, (o)=>{
			// return this.actionsMapNode(o)
			let attrs = {
				size:"large",
				title:o.label,
				key:o.code,
				icon:o.icon,
				type:o.primary ? "primary" : null,
				disabled:o.enableOnlyWhenSelected ? this.state.selectedRowKeys.length == 0 : false,
				onClick:this.actionsMapMethod(o.code),
			}
			let tips = (
				<Tooltip key={o.code} placement="bottom" title={o.name}>
					<Button {...attrs}/>
				</Tooltip>)

			return tips
		})

		// actions.push(<Tooltip key="abc" placement="bottom" title="test"><Button>abc</Button></Tooltip>);
		return actions;
	}

	renderToolBar(){
		const Group = Button.Group;
		let filtersAvailable = this.props.currentModel.columns.filter((col) => (
			col.field 
			&& col.field.hasFilterMethod 
			&& (!this.props.initFilter || (this.props.initFilter && col.field.path !== this.props.initFilter.filter)))
			|| col.type === 'heading' 
		)
		return(
			<div id="table-toolbar" className="btn-toolbar">

				<Group>

					{this.renderActions()}

				</Group>
				<ListToolBar
					model={this.props.currentModel}
					columns={this.props.columns}
					dispatch={this.props.store.dispatch}
					keepOn
					filtersActive={this.props.filters}
					filtersAvailable={filtersAvailable}
					refreshHandler={() => {
						this.props.store.dispatch(loadDataFromState());
					}}
				/>
			</div>
		);
	}
	renderFilters(){
		return <ListFilter
			dispatch={this.props.store.dispatch}
			filters={this.props.filters}
			hideFilters={this.props.initFilter ? [this.props.initFilter.filter] : []}
		/>
	}
	renderTable(){

		var columns = this.fields(this.props.columns);

		const handleTableChange = (pagination, filters, sorter) => {
			if (sorter.field){
				this.props.store.dispatch(setActiveSort((sorter.order == "descend" ? "-" : "+") + sorter.field));
			}
			this.props.store.dispatch(listPage({
				index:pagination.current,
				size:pagination.pageSize,
			}));
			this.props.store.dispatch(loadDataFromState());

		}


		const data = _.map(this.props.data, (o) => {
			o.key = o.id;
			return o;
		});
		const rowSelection = {
			selectedRowKeys:this.state.selectedRowKeys || [],
			onChange: this.onSelectionChange.bind(this),
		};
		return (
			<Table
				{...{
					columns,
					dataSource:data,
					loading:this.props.loading,
					rowSelection,
					pagination:{
						current:this.props.page.index,
						showSizeChanger:true,
						total:this.props.total,
						pageSize:this.props.page.size,
						showQuickJumper:true,
					},
					onChange:handleTableChange.bind(this),
				}}
			>
			</Table>

		);

	}
	renderAlertBox(){
		if (this.props.error){
			return <AlertBox  style="danger" title="错误" message={this.props.error} className="content-group"/>
		}
		return null;
	}
	editAfterCreatedHandler(data){
		var url = path.join(this.context.adminPath, "item", this.props.currentModel.key, data.id);
		this.context.router.push(url);
	}
	refreshAfterCreateHandler(e){
		this.props.store.dispatch(loadDataFromState());
	}
	refreshAfterAuthHandler(e){
		this.setState({
			selectedRowKeys:[]
		})
	}
	render(){
		if (!this.props.currentModel) return null;

		var node;
		var ids = this.state.selectedRowKeys;
		var data = this.props.data;
		var selections = _.filter(data, function(o) { return ids.indexOf(o.id) != -1 });

		const title = (
			<div>
				<h3 className="box-title">{this.props.title}</h3>
			</div>
		)
		return (
				<Card title={title} className="lte-box">

					<div>
						{this.renderAlertBox()}
						{this.renderToolBar()}
						{this.renderFilters()}
						{this.renderTable()}
					</div>
					<Modal ref="Modal"/>
					<Modal ref="RoleModal"/>
					<ItemCreateModal ref="createModal"
									 store={this.props.store}
									 model={this.props.currentModel}
									 isNeedCreateParams={this.props.isNeedCreateParams}
									 params={this.props.initFilter}
									 successAndEdit={this.editAfterCreatedHandler.bind(this)}
									 successAndClose={this.refreshAfterCreateHandler.bind(this)}

					/>
					<RoleAppointModal ref="roleappoint"
									  model={this.props.currentModel}
									  users={selections}
									  successAndClose={this.refreshAfterAuthHandler.bind(this)}

					/>
					<ModuleAppointModal ref="moduleappoint"
									  model={this.props.currentModel}
									  roles={selections}
									  successAndClose={this.refreshAfterAuthHandler.bind(this)}

					/>
					<ModelAppointModal ref="modelappoint"
									   model={this.props.currentModel}
									   roles={selections}
									   successAndClose={this.refreshAfterAuthHandler.bind(this)}

					/>
				</Card>
		);
	}
};
ListView.contextTypes = {
	router:React.PropTypes.object.isRequired,
	adminPath:React.PropTypes.string.isRequired,
}
export {
	ListView,
}
export default connect((state) => {
	return {
		selectedModule:state.app.selectedModule,
		loading:state.lists.loading,
		currentModel:state.lists.currentModel,
		page:state.lists.page,
		sort:state.active.sort,
		filters:state.active.filters,
		totalPages:state.lists.totalPages,
		total:state.lists.total,
		data:state.lists.data,
		pageSize:state.lists.pageSize,
		createData:state.item.createData,
		menuActions:state.lists.menuActions,
	}
})(ListView)


//KAdm.cateContent = ReactDOM.render(
//	<CateContent adminPath={KAdm.adminPath} apiPath="api2" model={KAdm.model.singular} navs={KAdm.navs} onTitleSelected={KAdm.control.cateContent.onTitleSelected} />, $("#cateContent")[0]
//);
