import React from 'react';
import assign from 'object-assign';
class TreeNode extends React.Component{
	
	constructor(props){
		super(props);
	}
	
	onSelect(e){
		this.props.onSelectHandler(e, this);
		e.stopPropagation();
	}
	onCheck(e){
		this.props.onCheckHandler(e, this);
		e.stopPropagation();
	}
	onExpand(e){
		this.props.onExpandHandler(e, this);
		e.stopPropagation();
	}
	render() {

		var node = this.props.node;
		var options = this.props.options;

		var style;

		if (options.highlightSelected && this.props.selected) {
			style = {
				color: options.selectedColor,
				backgroundColor: options.selectedBackColor
			};
		}
		else {
			style = {
				color: node.color || options.color,
				backgroundColor: node.backColor || options.backColor
			};
		}

		if (!options.showBorder) {
			style.border = 'none';
		}
		else if (options.borderColor) {
			style.border = '1px solid ' + options.borderColor;
		}
		
		var checkbox;
		
		if (this.props.showCheckbox)
		{
			var className = this.props.checked ? "icon check-icon glyphicon glyphicon-check" :
				"icon check-icon glyphicon glyphicon-unchecked";
			checkbox = (
				<span className={className} onClick={this.onCheck.bind(this)}></span>
			)
		}
		
		var indents = [];
		for (var i = 0; i < this.props.level-1; i++) {
			indents.push(<span className='indent' key={i}></span>);
		}

		var expandCollapseIcon;
		if (node.nodes) {
			if (!this.props.expanded) {
				expandCollapseIcon = (
					<span className={options.expandIcon}
						  onClick={this.onExpand.bind(this)}>
          </span>
				);
			}
			else {
				expandCollapseIcon = (
					<span className={options.collapseIcon}
						  onClick={this.onExpand.bind(this)}>
          </span>
				);
			}
		}
		else {
			expandCollapseIcon = (
				<span className={options.emptyIcon}></span>
			);
		}
		var nodeIcon  = this.showIcon ? (
			<span className='icon'>
        		<i className={node.icon || options.nodeIcon}></i>
      		</span>
		) : <span  className='indent'/>;

		var nodeText;
		if (options.enableLinks) {
			nodeText = (
				<a href={node.href} /*style="color:inherit;"*/>
					{node.text}
				</a>
			);
		}
		else {
			nodeText = (
				<span>{node.text}</span>
			);
		}

		var badges;
		if (options.showTags && node.tags) {
			badges = node.tags.map(function (tag) {
				return (
					<span className='badge'>{tag}</span>
				);
			});
		}

		return (
			<li className='list-group-item'
				style={style}
				data-nodeId={this.props.node.nodeId}
				onClick={this.onSelect.bind(this)}
			>
				{indents}
				{expandCollapseIcon}
				{checkbox}
				{nodeIcon}
				{nodeText}
				{badges}
			</li>
		);
	}
};

export default TreeNode;

//export default TreeNode;
//export {
//	TreeNode
//}
//export default connect((state, ownProps) => {
//	var key = "treenode" + ownProps.nodeId;
//	var prop = {
//		selected:state.treeview.selectedNodeId == ownProps.nodeId,
//	}
//	var nodeState = state.treeview.treeNodes[key];
//	
//	if (nodeState)
//	{
//		prop = assign({}, prop, nodeState)
//	}
//	return prop;
//	
//})(TreeNode)
