/**
 * Item View
 *
 * This is the item view, it is rendered when users visit a page of a specific
 * item. This mainly renders the form to edit the item content in.
 */

import React from 'react';
import { Container, Spinner } from 'elemental';
import { connect } from 'react-redux';
import { Link } from 'react-router';


import {
	actions,
    components,
    utils,
} from 'kadm-component';
var { listsByKey } = utils.lists;
var {
	EditForm,
	RelatedItemsList
} = components
var {
	Loading,
} = components.utilsForReactDom;

var {
	itemDataLoaded,
	itemDataLoadingError,
	itemDeleteItem,
	loadItemData,
} = actions.item;

var {
	selectModel,
	selectList,
} = actions.lists;

var ContentHeader = components.ContentHeader;
import {
	Alert,
    Layout,
    Card,
} from 'antdlib'
const { Header, Content } = Layout;
import PropTypes from 'prop-types'
//import {
//	selectList,
//} from 'keystone/admin/client/App/screens/List/actions';

class ItemView extends React.Component{
	constructor(props){
		super(props)
		this.displayName = 'ItemView',
		this.contextTypes = {
			router: PropTypes.object.isRequired,
		}
	}
	getInitialState () {
		return {
			createIsOpen: false,
		};
	}
	componentWillMount () {
		// When we directly navigate to an item without coming from another client
		// side routed page before, we need to select the list before initializing the item
		this.props.dispatch(selectModel(this.props.params.model));
		this.initializeItem(this.props.params.id);

	}
	componentWillReceiveProps (nextProps) {
		// We've opened a new item from the client side routing, so initialize
		// again with the new item id
		if (nextProps.params.id !== this.props.params.id) {
			this.props.dispatch(selectModel(nextProps.params.model));
			this.initializeItem(nextProps.params.id);
		}
	}
	// Initialize an item
	initializeItem (itemId) {
		this.props.dispatch(loadItemData(itemId))
	}
	// Called when a new item is created
	onCreate (item) {
		// Hide the create form
		this.setState({
			createIsOpen: false,
		});
		// Redirect to newly created item path
		const list = this.props.currentModel;
		this.context.router.push(`${Keystone.adminPath}/${list.path}/${item.id}`);
	}
	// Open and close the create new item modal
	toggleCreate (visible) {
		this.setState({
			createIsOpen: visible,
		});
	}
	// Render this items relationships
	renderRelationships () {
		const { relationships } = this.props.currentModel;
		const keys = Object.keys(relationships);
		if (!keys.length) return;
		return (
			<div className="Relationships">
				<Container>
					<h2>Relationships</h2>
					{keys.map(key => {
						const relationship = relationships[key];
						const refList = listsByKey[relationship.ref];
						return (
							<RelatedItemsList
								key={relationship.path}
								list={this.props.currentModel}
								refList={refList}
								relatedItemId={this.props.params.id}
								relationship={relationship}
							/>
						);
					})}
				</Container>
			</div>
		);
	}
	// Handle errors
	handleError (error) {
		const detail = error.detail;
		if (detail) {
			// Item not found
			if (detail.name === 'CastError'
				&& detail.path === '_id') {
				return (
					<Container>
						<p>Item not found!</p>
						<Link onClick={(e) => {
							this.context.router.goBack();
						}}>
							返回
						</Link>
					</Container>
				);
			}
		}
		if (error.message) {
			// Server down + possibly other errors
			if (error.message === 'Internal XMLHttpRequest Error') {
				return (
					<Container>
						<p>We encountered some network problems, please try refreshing!</p>
					</Container>
				);
			}
		}
		return (<p>Error!</p>);
	}
	render () {
		// If we don't have any data yet, show the loading indicator
		if (this.props.loading) {
			return (
				<Loading/>
			);
		}
		// When we have the data, render the item view with it

		if (this.props.data && this.props.data.error){
			// has not permission to load and show item.
			//TODO it must be react as a error page in furchur.

			return <Alert
				message="Error"
				description={this.props.data.error}
				type="error"
				showIcon
			/>;
		}


        const title = (
			<div>
				<h3 className="box-title">{this.props.selectedModule ? this.props.selectedModule.name : ""}</h3>
			</div>
		)
		return (
            <Card title={title} className="lte-box">
                <div className="itemScene" data-screen-id="item">
                    {(this.props.error) ? this.handleError(this.props.error) : (
                        <div>
                            <Container>
                                <EditForm
                                    adminPath={KAdm.adminPath}
                                    list={this.props.currentModel}
                                    data={this.props.data}
                                    dispatch={this.props.dispatch}
                                    router={this.context.router}
                                />
                            </Container>
                            {this.renderRelationships()}
                        </div>
                    )}
                </div>
            </Card>
		);
	}
};
export {
	ItemView
}
export default connect((state) => ({
	data: state.item.data,
	loading: state.item.loading,
	ready: state.item.ready,
	error: state.item.error,
    currentModel: state.lists.currentModel,
    selectedModule:state.innerapp.selectedModule,
    
}))(ItemView);
