var series = require('async/series');

var Base = require('../../api/global/Base');
var kadm = require('../../../../');
var _ = require('lodash');
var rbacManager = require('../../../../core/acl/RBACManager');
var keystone = kadm.getAdminPlus();

/**
 *
 * @param req
 * @param res
 *
 * used with model 'UserRole' only
 */

module.exports = function (req, res) {
	var data = (req.method === 'POST') ? req.body : req.query;
	if (!keystone.security.csrf.validate(req)) {
		return Base.error(res, new Error('csrf检验错误'));
	}

	var action = data.action;


	switch(action)
	{
		case "get_permissions":{
			let {
				role
			} = req.query;

			if (role){
				let valueMap = {};
				rbacManager.rbac.getScope(role, (err, permissions) => {
					if (err) {
						Base.error(res, err);
					} else {

						_.map(permissions, (o) => {
							let params = o.split('_');
							resource = params[1];
							action = params[0];
							valueMap[resource] = valueMap[resource] ? valueMap[resource] : [];
							valueMap[resource].push(action);
						});
						Base.json(res, {
							status:1,
							info:valueMap,
						});
					}
				})
			}else {
				Base.error(res, new Error("没有找到用户"));
			}
		}
		break;
		case "auth_module":
		{

			/**
			 *
			 * 在router 中计算权限
			 *
			 */
			let {
				role,
				valueMap
			} = req.body;


			let grants = [];
			valueMap = JSON.parse(valueMap);

			_.forEach(valueMap, (value, key) => {
				grants = _.concat(grants, _.map(valueMap[key], (o) => o + "_" + key))
			});

			const task = {
				role:(cb) => rbacManager.rbac.getRole(role, cb),
				revokes:(cb) => {

					//remove all exists permissions
					rbacManager.rbac.getScope(role, (err, permissions) => {
						if (err) {
							cb(err, null);
						} else {
							let queue = _.map(permissions, (o) => {
								return (cb2) => rbacManager.rbac.revokeByName(role, o, cb2)
							});
							series(queue, cb)
						}
					})

				},
				permissions:(cb) =>{
					//examinte all permissions whether need create;
					let permisions = [];
					_.forEach(valueMap, (value, key) => {
					
						_.map(value, (action) => {
							permisions = _.concat(permisions,
								(cb2) => rbacManager.rbac.getPermission(action, key, (err, permission) => {

									if (permission) cb2(null, permission)
									else {
										rbacManager.rbac.createPermission(action, key, true, (err, items) => {
											cb2(err, items)
										})
									}
								})
							);

						})
					})
					series(permisions, cb);

				}
			}

			series(task, (err, result) => {
				if (result.role && result.permissions){
					rbacManager.rbac.grants({
						[result.role.name]:grants,
					}, (err, items) => {
						if (err) return Base.error(res, "error", err, err.message);
						Base.json(res, {
							status:1,
							list:_.map(result.permissions, (o) => o.name),
						});

					})
				}
			})


		}
			break;
		case "updateAndCreateWithNames":
		{

			//use with auth user with roles name

			if (req.list.key != 'UserRole'){
				return Base.error(res, "error", new Error("本接口只使用UserRole"), "");
			}
			let whereCase = data.whereCase || {};
			if (typeof whereCase == "string") whereCase = JSON.parse(whereCase);
			data.role =  JSON.parse(data.role);
			var updateData = _.assign({}, data);

			let cases = whereCase;
			let fns = _.map(cases, (name) => {
				//for each user auth we check whether there already exist
				return (cb) => {
					req.list.model.find({
						name,
					}).exec((err, item)=>{
						if (err || item.length == 0){
							// if not exits , creat
							let newItem = new req.list.model();
							newItem = _.assign(newItem, {name,});
							req.list.updateItem(newItem, updateData, (err)=>{
								if (err) cb(err, 0)
								else cb(null, 1)
							})
						}
						else {
							cb(null, 1)
						}
					})
				}
			});
			series(fns, (err, result) => {
				if (err || _.sum(result) != result.length){
					//not all use check created or exist
					return Base.error(res, "error", err, "新增的队例不符或出错");
				}
				else {
					//finally, we update all auth record together
					req.list.model.update({
						name:{$in:cases}
					},{
						$set: updateData,
					}, {
						upsert:true,
						multi:true,
						setDefaultsOnInsert:true,
					},(err, result) => {
						if (err) return Base.error(res, "error", err, err.message);
						req.list.model.find({
							name:{$in:whereCase}
						}).exec((err, item)=>{
							if (err) return Base.error(res, "error", err, err.message);
							if (!item || item.length == 0) return Base.error(res, "error", new Error("item not found"), "item not found");
							Base.json(res, {
								status:1,
								list:item
							});
						})
					});
				}
			})

		}

		break;

		default:
			Base.error(res, new Error("不合适的接合指令"));
			break;

	}
};
