
/**
 * Created by kinfen on 16/3/5.
 */
var Base = require('../api/global/Base');
var kadm = require('../../../');
var _ = require('lodash');
var async = require('async');
var keystone = kadm.getAdminPlus();

var renderJson = function(req, res, item) {


	var fields = req.query.fields;
	if (fields === 'false') {
		fields = false;
	}
	if (typeof fields === 'string') {
		fields = listToArray(fields);
	}
	if (fields && !Array.isArray(fields)) {
		return res.status(401).json({ error: 'fields must be undefined, a string, or an array' });
	}


	var tasks = [];
	var drilldown;

	/* Drilldown (optional, provided if ?drilldown=true in querystring) */
	if (req.query.drilldown === 'true' && req.list.get('drilldown')) {
		drilldown = {
			def: req.list.get('drilldown'),
			items: [],
		};

		tasks.push(function (cb) {

			// TODO: proper support for nested relationships in drilldown

			// step back through the drilldown list and load in reverse order to support nested relationships
			drilldown.def = drilldown.def.split(' ').reverse();

			async.eachSeries(drilldown.def, function (path, done) {

				var field = req.list.fields[path];

				if (!field || field.type !== 'relationship') {
					throw new Error('Drilldown for ' + req.list.key + ' is invalid: field at path ' + path + ' is not a relationship.');
				}

				var refList = field.refList;

				if (field.many) {
					if (!item.get(field.path).length) {
						return done();
					}
					refList.model.find().where('_id').in(item.get(field.path)).limit(4).exec(function (err, results) {
						if (err || !results) {
							done(err);
						}
						var more = (results.length === 4) ? results.pop() : false;
						if (results.length) {
							// drilldown.data[path] = results;
							drilldown.items.push({
								list: refList.getOptions(),
								items: _.map(results, function (i) {
									return {
										label: refList.getDocumentName(i),
										href: '/' + keystone.get('admin path') + '/' + refList.path + '/' + i.id,
									};
								}),
								more: (more) ? true : false,
							});
						}
						done();
					});
				} else {
					if (!item.get(field.path)) {
						return done();
					}
					refList.model.findById(item.get(field.path)).exec(function (err, result) {
						if (result) {
							// drilldown.data[path] = result;
							drilldown.items.push({
								list: refList.getOptions(),
								items: [{
									label: refList.getDocumentName(result),
									href: '/' + keystone.get('admin path') + '/' + refList.path + '/' + result.id,
								}],
							});
						}
						done(err);
					});
				}

			}, function (err) {
				// put the drilldown list back in the right order
				drilldown.def.reverse();
				drilldown.items.reverse();
				cb(err);
			});

		});
	}

	/* Process tasks & return */
	async.parallel(tasks, function (err) {
		if (err) {
			return res.status(500).json({
				err: 'database error',
				detail: err,
			});
		}
		res.json(_.assign(req.list.getData(item, fields), {
			drilldown: drilldown,
		}));
	});
};
module.exports = function(req, res) {

	var data = (req.method === 'POST') ? req.body : req.query;
	if (!keystone.security.csrf.validate(req)) {
		return Base.error(res, "Error", new Error('csrf检验错误'), 'csrf检验错误');
	}
	var action = data.action;
	switch(action)
	{
		case "create":
		{
			var item = new req.list.model();
			var updateData = _.assign({}, data, req.files);

			req.list.updateItem(item, updateData, function (err) {
				if (err) {
					return Base.error(res, "Error", err, err.message);
				} else {
					Base.json(res, {
						status:1,
						info:{
							item:req.list.getData(item)
						}
					});
				}
			});

		}
			break;
		case "delete":
		{
			if (req.list.get('nodelete')) {
				console.log(`Refusing to delete ${req.list.key} items; List.nodelete is true`);
				return Base.error(res, "Error", new Error('指定的类型不能删除'), '指定的类型不能删除');
			}
			var ids = req.body.ids || req.body.id || req.params.id;
			if (typeof ids === 'string') {
				ids = ids.split(',');
			}
			if (!Array.isArray(ids)) {
				ids = [ids];
			}
			if (req.user) {
				var userId = String(req.user.id);
				if (ids.some(function(id) {
						return id === userId;
					})) {
					console.log(`Refusing to delete ${req.list.key} items; ids contains current User`);
					return Base.error(res, "Error", new Error('不能删除自身用户'), '不能删除自身用户');
				}
			}
			var deletedCount = 0;
			var deletedIds = [];
			req.list.model.find().where('_id').in(ids).exec(function (err, results) {
				if (err) {
					console.log(`Error deleting ${req.list.key} items:`, err);
					return Base.error(res, "Error", err, err.message);
				}
				async.forEachLimit(results, 10, function(item, next) {
					item.remove(function (err) {
						if (err) return next(err);
						deletedCount++;
						deletedIds.push(item.id);
						next();
					});
				}, function() {
					return Base.json(res, {
						status:1,
						delete_ids: deletedIds,
						count: deletedCount
					});
				});
			});
		}
			break;
		case "get":
		{
			var itemQuery = req.list.model.findById(req.params.id).select();

			itemQuery.exec(function(err, item) {

				if (err) {
					return Base.error(res, "Error", err, err.message);
				}

				if (!item) {
					var msg = 'Item ' + req.params.item + ' could not be found.';
					return Base.error(res, "Error", new Error(msg), msg);
				}

				return renderJson(req,res, item);


			});
		}
			break;
		case "update":
		{
			let whereCase = data.whereCase || {};
			if (typeof whereCase == "string") whereCase = JSON.parse(whereCase);
			var updateData = _.assign({}, data, req.files);

			req.list.model.update(whereCase,{
				$set: updateData,
			}, {
				upsert:true,
				multi:true,
				setDefaultsOnInsert:true,
			},(err, res) => {
				if (err) return Base.error(res, "error", err, err.message);
				req.list.model.find(whereCase).exec((err, item)=>{
					if (err) return Base.error(res, "error", err, err.message);
					if (!item || item.length == 0) return Base.error(res, "error", new Error("item not found"), "item not found");
					Base.json(res, {
						status:1,
						info:item
					});
				})
			});
		}

	}


};




