/**
 * Created by kinfen on 16/1/19.
 * 处理所有后台路由
 */
var	express = require('express');
var kadm = require('../../../index');
var keystone = kadm.getAdminPlus();
var path = require('path');
var series = require('async/series');
var rbacManager = require('../../../core/acl/RBACManager');
var _ = require('lodash');


let kadmAuth = (req, res, next) => {
	if (!req.user || !req.user.canAccessKeystone) {
		if (req.headers.accept === 'application/json') {
			return req.user
				? res.status(403).json({error: 'not authorised'})
				: res.status(401).json({error: 'not signed in'});
		}
		var regex = new RegExp('^\/' + kadm.get('kadmPath') + '\/?$', 'i');
		var from = regex.test(req.originalUrl) ? '' : '?from=' + req.originalUrl;
		return res.redirect(kadm.get('signin url') + from);
	}

	next();

}

exports = module.exports = function() {
	var router = express.Router();
	var keystonePath = kadm.getKeystoneRelativePath(__dirname);

	// Bind the request to the keystone instance
	router.use(function (req, res, next) {
		req.keystone = keystone;
		next();
	});

	// #1: Session API
	//// TODO: this should respect keystone auth options
	router.get('/api/session', require(path.join(keystonePath, 'keystone/admin/server/api/session/get')));
	router.post('/api/session/signin', require(path.join(keystonePath, 'keystone/admin/server/api/session/signin')));
	router.post('/api/session/signout', require(path.join(keystonePath, 'keystone/admin/server/api/session/signout')));


	let SigninRender = require('./middleware/siginRender')
	let SignoutRender = require('./middleware/sigoutRender');

	//auth
	if (keystone.get('auth') === true) {
		// TODO: poor separation of concerns; settings should be defaulted elsewhere
		if (!kadm.get('signout url')) {
			kadm.set('signout url', '/' + kadm.get('kadmPath') + '/signout');
		}
		if (!kadm.get('signin url')) {
			kadm.set('signin url', '/' + kadm.get('kadmPath') + '/signin');
		}
		if (!keystone.nativeApp || !keystone.get('session')) {
			router.all('*', keystone.session.persist);
		}
		router.all('/signin', SigninRender(keystone));
		router.all('/signout', SignoutRender(keystone));
		router.use(kadmAuth);
	} else if ('function' === typeof keystone.get('auth')) {
		router.use(keystone.get('auth'));
	}

	return router;
};
