/**
 * Created by kinfen on 16/7/25.
 */
var _ = require('lodash');
var ejs = require('ejs');
var path = require('path');
var {
	seq,
	series,
} = require('async');
var kadm = require('../../../../');
var templatePath = path.resolve(__dirname, '../../templates/views/admin.ejs');
var config = require('../../../../config/config');
var rbacManager = require('../../../../core/acl/RBACManager');
module.exports = function AdminRender (keystone) {
	return function (req, res) {

		var lists = {};
		_.forEach(keystone.lists, function (list, key) {
			lists[key] = list.getOptions();
			list.expandOptions(lists[key]);
		});

		var UserList = keystone.list(keystone.get('user model'));
		var orphanedLists = keystone.getOrphanedLists().map(function (list) {
			return _.pick(list, ['key', 'label', 'path']);
		});
		
		var keystoneData = {
			adminCore: '/' + keystone.get('admin path'),
			adminPath: '/' + kadm.get("kadmPath"),
			appversion: keystone.get('appversion'),
			backUrl: keystone.get('back url') || '/',
			brand: keystone.get('brand'),
			csrf: { header: {} },
			devMode: !!process.env.KEYSTONE_DEV,
			lists: lists,
			nav: keystone.nav,
			orphanedLists: orphanedLists,
			signoutUrl: keystone.get('signout url'),
			user: {
				id: req.user.id,
				name: UserList.getDocumentName(req.user),
			},
			userList: UserList.key,
			version: keystone.version,
			wysiwyg: { options: keystone.get('wysiwyg option') || {
				menubar: false,
				height: 500,
				theme: 'modern',
				plugins: [
					'advlist autolink lists link image charmap print preview hr anchor pagebreak',
					'searchreplace wordcount visualblocks visualchars code fullscreen',
					'insertdatetime media nonbreaking save table contextmenu directionality',
					'emoticons template paste textcolor colorpicker textpattern imagetools codesample toc help'
				],
				toolbar1: 'undo redo | insert | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image',
				toolbar2: 'preview media | forecolor backcolor emoticons | codesample help',
				image_advtab: true,
			} },
		};
		keystoneData.csrf.header[keystone.security.csrf.CSRF_HEADER_KEY] = keystone.security.csrf.getToken(req, res);

		var codemirrorPath = keystone.get('codemirror url path')
			? '/' + keystone.get('codemirror url path')
			: '/' + keystone.get('admin path') + '/js/lib/codemirror';

		var pluginPath = _.map(kadm.getAdminPlus().get('plugins'), (o) => {
			var pluginPath = o.getWebPluginPath();
			return pluginPath ? path.join('/admin', o.getName(), 'js', 'plugin.js') : null;
		});
		_.pull(pluginPath, null);
		var locals = {
			adminCore: keystoneData.adminCore,
			adminPath: keystoneData.adminPath,
			cloudinaryScript: false,
			codemirrorPath: codemirrorPath,
			env: keystone.get('env'),
			fieldTypes: keystone.fieldTypes,
			ga: {
				property: keystone.get('ga property'),
				domain: keystone.get('ga domain'),
			},
			keystone: keystoneData,
			title: keystone.get('name') || 'Keystone',
			actions:[],
			pluginPath,
		};

		var cloudinaryConfig = keystone.get('cloudinary config');
		if (cloudinaryConfig) {
			var cloudinary = require('cloudinary');
			var cloudinaryUpload = cloudinary.uploader.direct_upload();
			keystone.cloudinary = {
				cloud_name: keystone.get('cloudinary config').cloud_name,
				api_key: keystone.get('cloudinary config').api_key,
				timestamp: cloudinaryUpload.hidden_fields.timestamp,
				signature: cloudinaryUpload.hidden_fields.signature,
			};
			locals.cloudinaryScript = cloudinary.cloudinary_js_config();
		};

		let task = {
			actions:(cb) => {
				let List = keystone.list('Action');
				List.model.find({
					state:{$eq:1},
				}).exec(cb)
			}
		}
		series(task, (err, results) => {
			if (err) locals.actions = [];
			locals.actions = results.actions;
			locals.permissions = req.permissions;
			locals.isCreator = req.isCreator || false;
			locals.roles = req.roles;
			locals.idAuthModel = rbacManager.idAuthModel;
			ejs.renderFile(templatePath, locals, {}, function (err, str) {
				if (err) {
					console.error('Could not render Admin UI Index Template:', err);
					return res.status(500).send(keystone.wrapHTMLError('Error Rendering Admin UI', err.message));
				}
				res.send(str);
			});
		})

	};
};
