

/**
 * Created by kinfen on 2017/5/10.
 */

var RBAC = require('rbac').default;
var Base = require('.././../admin/server/api/global/Base');
var {
	Mongoose
} = require('rbac');
var kadm = require("../../index");
var keystone = kadm.getAdminPlus();
var _ = require('lodash');
var {
	seq,
	series,
} = require('async');

var path = require('path');
var utils = require('keystone-utils');


class RBACManager {
	constructor(){

		this._rbac = new RBAC({
			storage:new Mongoose(keystone.mongoose)
		});
		// console.log(utils.plural('delete_5a77b926249e6205ab148483@modules'))
	}
	set currentUser(user){
		this.cuser = user;
	}
	get currentUser (){
		return this.cuser;
	}
	addPermissionWithRole(role, action, resource, callback){

		seq(
			(cb) => {

				//check role's permission exist
				rbacManager.rbac.getScope(role, (err, permissions) => {
					if (err) {
						cb(err, null);
					} else {
						_.forEach(permissions, (o) => {
							if( o ==  `${actin}_${resource}`)
								cb(null, o)

						});
						cb (null, null);
					}
				})

			},

			(existPermission, cb) => {

				if (!existPermission){

					//create permission if role has no that permission
					rbacManager.rbac.getPermission(action, resource, (err, permission) => {

						if (permission) cb(null, permission)
						else {
							rbacManager.rbac.createPermission(action, key, true, cb)
						}
					})
				}
				else {
					rbacManager.rbac.getPermission(action, resource, cb);
				}
			},

		)(role, (err, permission) => {
			if (err) callback(err, null);
			rbacManager.rbac.grantByName(role, permission.name, callback);
		});

	}


	permissionsMapWithUser(user, cb){
		this.rolesWithUser(user, (err, roles) => {

			//if is root?

			let nameQueue = _.map(roles, (role) => {
				return role.code
			})

			if (nameQueue.indexOf('root') != -1){
				//assign all the permissions
				let map = {};

				let List = keystone.list('Action');
				List.model.find({
					state:{$eq:1},
				}).exec((err, results) => {
					_.forEach(keystone.lists, (value, key) => {
						map[key] = _.map(results, (o) => o.code);

					})
					cb(null, map);
				});


			} else {
				let queue = _.map(roles, (role) => {
					return (cb) => this._rbac.getScope(role.name, cb)
				})

				series(queue, (err, results) => {
					let lastres = [];
					_.forEach(results, (o) => {
						lastres = _.concat(lastres, o);
					})

					let map = {};
					_.forEach(lastres, (o) => {
						let params = o.split('_');
						let resource = params[1];
						let action = params[0];

						map[resource] = map[resource] ? map[resource] : [];
						if (map[resource].indexOf(action) == -1) {
							map[resource].push(action);
						}

					})
					cb(err, map);

				})
			}

		})
	}
	rolesWithUser(user, cb){
		let List = keystone.list('UserRole');
		List.model.find().where({
			name:user
		}).populate({
			path:'name',
			model:'User'
		}).populate('role').exec((err, result) => {
			cb (err, result && result.length > 0 ? result[0].role : null)
		})
	}


	get rbac(){
		return this._rbac
	}

	bindRouterParams(targetRouter) {

		let paramList = ['treemodel', 'model', 'list'];

		let idList = ['id', 'item'];

		_.forEach(paramList, (o) => targetRouter.param(o, (req, res, next, value) => {

			//recover the model's key from its path

			if (req.roles.indexOf('root') > -1)
				return next();

			let words =  utils.singular(value).split('-');
			words = _.map(words, (o) => {
				return o.substring(0,1).toUpperCase() + o.substring(1, o.length)
			})
			let word = words.join('');

			let {permissions} = req;
			if (!permissions) Base.error(res,  new Error(`缺少${word}的权限`));

			if (!(permissions[value] || permissions[word] || permissions[utils.plural(word)]) ){
				return Base.error(res,  new Error(`缺少${word}的权限`));
			}
			req.lastAuthModel = word;
			next();

		}));

		_.forEach(idList, (o) => targetRouter.param(o, (req, res, next, value) => {

			//recover the model's key from its path
			if (req.roles.indexOf('root') > -1)
				return next();

			let {
				lastAuthModel,
				permissions
			} = req;

			if (!lastAuthModel || ! permissions) Base.error(res,  new Error(`找不到授权的model及permission`));

			

			if (this.idAuthModel.indexOf(lastAuthModel) != -1 && utils.isValidObjectId(value)){

				//only use when value is objectid, prevent case like /list/create
				let list = keystone.list(lastAuthModel);
				list.model.find({
					_id:value,
					createdBy:req.user.id,
				}).exec((err, items) => {
					//let it go if that id created by current user.
					if (!err && items.length > 0)
						req.isCreator = true;
						return next();

					if (!permissions[value + "@" + lastAuthModel]){
						return Base.error(res,  new Error(`缺少id为${value}的权限`));
					}
					//id有授权，通过
					next();
				})
			}
			else {
				//非id授权项目，直接通过
				next();
			}

		}));
	}

	fixResourceName(originName){
		let words =  utils.singular(originName).split('-');
		words = _.map(words, (o) => {
			return o.substring(0,1).toUpperCase() + o.substring(1, o.length)
		})
		return words.join('');
	}

	routerCheckWithParamName(action, paramName){
		return (req, res, next) => {

			let pName;
			if (Array.isArray(paramName) && paramName.length == 2){
				//存在id的检查方法
				let resourceName = this.fixResourceName( req.params[paramName[1]]);

				if (this.idAuthModel.indexOf(resourceName) != -1 && req.params[paramName[0]]
				 && utils.isValidObjectId(req.params[paramName[0]]))
					pName = req.params[paramName[0]] + "@" + resourceName;
				else
					pName = resourceName;
			}
			else if (typeof paramName == 'string'){
				pName = this.fixResourceName( req.params[paramName]);
			}
			this.routerCheckWithModel(action, pName)(req, res, next);

		}
	}

	routerCheckWithModel(action, model) {
		return (req, res, next) => {

			// return Base.error(res, new Error('直接禁止'));

			if (req.roles.indexOf('root') != -1 || req.isCreator)
				return next();

			if (req.permissions && req.permissions[model].indexOf(action) != -1)
				return next();

			//TODO 后期统一redirect到错误页面，目前无法统一有效处理错误返回方式;
			return Base.error(res, new Error(`${model}缺少权限${action}`));

		}
	}

	dbFilterWithIdAuth(req, db, modelName, cb){

		let {permissions, roles} = req;

		if (roles.indexOf('root') != -1)
			return cb(null, db);

		if (permissions[modelName] && permissions[modelName].indexOf('read') != -1){
			let ids = _.map(_.keys(permissions), (o) => {
				let array = o.split('@');
				return array[0]
			})
			if (this.idAuthModel.indexOf( modelName) != -1){
				ids = _.filter(ids, (o) => utils.isValidObjectId(o));
				// _.pullAll(ids,_.keys(keystone.lists));
				db = db.or([
					{
						_id:{$in:ids}
					},
					{
						createdBy:req.user.id,
					}
				]);
			}
		}
		cb(null, db);
	}
	bindPermissionToRequest(req, res, next){
		let task = {
			roles:(cb) => this.rolesWithUser(req.user.id, cb),
			permissions:(cb) => this.permissionsMapWithUser(req.user.id, cb),
		}
		series(task, (err, result) => {
			let nameQueue = _.map(result.roles, (role) => {
				return role.code
			})
			req.roles = nameQueue;
			req.permissions = result.permissions;
			next();
		})
	}

	get idAuthModel(){
		return ['Module']
	}


}


exports = module.exports = new RBACManager();
