import classnames from 'classnames';
import evalDependsOn from '../utils/evalDependsOn.js';
import React from 'react';
import { findDOMNode } from 'react-dom';
 //import { FormField, FormInput, FormNote } from 'elemental';

import {Form,Input} from 'antdlib';

import blacklist from 'blacklist';
import CollapsedFieldLabel from '../components/CollapsedFieldLabel';

function isObject (arg) {
	return Object.prototype.toString.call(arg) === '[object Object]';
}

function validateSpec (spec) {
	if (!spec) spec = {};
	if (!isObject(spec.supports)) {
		spec.supports = {};
	}
	if (!spec.focusTargetRef) {
		spec.focusTargetRef = 'focusTarget';
	}
	return spec;
}

var Base = module.exports.Base = {
	fieldSize: "large",
	getInitialState () {
		return {};
	},
	getDefaultProps () {
		return {
			adminPath: Keystone.adminPath,
			inputProps: {},
			labelProps: {},
			valueProps: {},
		};
	},
	getInputName (path) {
		// This correctly creates the path for field inputs, and supports the
		// inputNamePrefix prop that is required for nested fields to work
		return this.props.inputNamePrefix
			? `${this.props.inputNamePrefix}[${path}]`
			: path;
	},
	valueChanged (event) {
		// this.props.onChange({
		// 	path: this.props.path,
		// 	value: event.target.value,
		// });
	},
	shouldCollapse () {
		return this.props.collapse && !this.props.value;
	},
	shouldRenderField () {
		if (this.props.mode === 'create') return true;
		return !this.props.noedit;
	},
	/**
	 *  //whether we use ant field decorator to check field state,
	 *  set false when we render a not input style compenent such as Button
	 * @returns {boolean}
     */
	shouldUseFieldDecorator(){
		return true;
	},
	focus () {
		if (!this.refs[this.spec.focusTargetRef]) return;
		findDOMNode(this.refs[this.spec.focusTargetRef]).focus();
	},
	renderNote () {
		if (!this.props.note) return null;
		return <p className="ant-form-explain">{$(this.props.note).text()}</p>
		//return <FormNote note={this.props.note} />;
	},
	getValidateRules(){
		const { required } = this.props;
		return [{ required, message: '请输入' + this.props.label + '!' }];
	},
	renderField () {

		const { autoFocus, inputProps, path } = this.props;
		return (
			//now we use ant input instead of that one in elements
			<Input
				{...{
					...inputProps,
					autoFocus,
					name: this.getInputName(path),
					size:this.fieldSize,
				}}

			/>
		);
	},
	renderValue () {
		return <Input disabled value={this.props.value}></Input>;
	},
	getInitValue(){
		const {  value,   defaultValue} = this.props;
		return value == null ? defaultValue : value;
	},
	renderUI () {
		var wrapperClassName = classnames(
			'field-type-' + this.props.type,
			this.props.className,
			{ 'field-monospace': this.props.monospace }
		);
		let valueNode = null;

		let shouldRenderField = this.shouldRenderField();
		if (shouldRenderField){
			const {   path} = this.props;
			const { getFieldDecorator } = this.props.form;

			valueNode = this.shouldUseFieldDecorator() ? getFieldDecorator(
				path,
				{
					rules: this.getValidateRules(),
					initialValue:this.getInitValue(),
				}
			)(
				this.renderField()
			) : this.renderField();
		} else{

			valueNode = this.renderValue();
		}

		return (
			<Form.Item
				key={this.props.key}
				label={this.props.label}
				{...this.props.formItemLayout}
				className={wrapperClassName}
				htmlFor={this.props.path}
				labelCol={{span:6}}
				wrapperCol={{span:18}}
			>
					{valueNode}
				{this.renderNote()}
			</Form.Item>
		);
	},
};

var Mixins = module.exports.Mixins = {
	Collapse: {
		componentWillMount () {
			this.setState({
				isCollapsed: this.shouldCollapse(),
			});
		},
		componentDidUpdate (prevProps, prevState) {
			if (prevState.isCollapsed && !this.state.isCollapsed) {
				this.focus();
			}
		},
		uncollapse () {
			this.setState({
				isCollapsed: false,
			});
		},
		renderCollapse () {
			if (!this.shouldRenderField()) return null;
			return (
				<FormField>
					<CollapsedFieldLabel onClick={this.uncollapse}>+ Add {this.props.label.toLowerCase()}</CollapsedFieldLabel>
				</FormField>
			);
		},
	},
};

module.exports.create = function (spec) {

	spec = validateSpec(spec);

	var field = {
		spec: spec,
		displayName: spec.displayName,
		mixins: [Mixins.Collapse],
		statics: {
			getDefaultValue: function (field) {
				return field.defaultValue || '';
			},
		},
		render () {
			if (!evalDependsOn(this.props.dependsOn, this.props.values)) {
				return null;
			}
			if (this.state.isCollapsed) {
				return this.renderCollapse();
			}
			return this.renderUI();
		},
	};

	if (spec.statics) {
		Object.assign(field.statics, spec.statics);
	}

	var excludeBaseMethods = {};
	if (spec.mixins) {
		spec.mixins.forEach(function (mixin) {
			Object.keys(mixin).forEach(function (name) {
				if (Base[name]) {
					excludeBaseMethods[name] = true;
				}
			});
		});
	}

	Object.assign(field, blacklist(Base, excludeBaseMethods));
	Object.assign(field, blacklist(spec, 'mixins', 'statics'));

	if (Array.isArray(spec.mixins)) {
		field.mixins = field.mixins.concat(spec.mixins);
	}

	return React.createClass(field);

};
