import _ from 'lodash';
import CodeMirror from 'codemirror';
import Field from '../Field';
import React from 'react';
import { findDOMNode } from 'react-dom';
import {Input} from 'antdlib';
import classnames from 'classnames';

/**
 * TODO:
 * - Remove dependency on lodash
 */

// See CodeMirror docs for API:
// http://codemirror.net/doc/manual.html

module.exports = Field.create({
	displayName: 'CodeField',
	statics: {
		type: 'Code',
	},

	getInitialState () {
		return {
			isFocused: false,
		};
	},
	componentDidMount () {
		if (!this.refs.codemirror) {
			return;
		}

		var options = _.defaults({}, this.props.editor, {
			lineNumbers: true,
			readOnly: this.shouldRenderField() ? false : true,
		});
		this.codeMirror = CodeMirror.fromTextArea(findDOMNode(this.refs.codemirror), options);
		this.codeMirror.setSize(null, this.props.height);
		this.codeMirror.on('change', this.codemirrorValueChanged);
		this.codeMirror.on('focus', this.focusChanged.bind(this, true));
		this.codeMirror.on('blur', this.focusChanged.bind(this, false));
		var value = this.props.form.getFieldValue(this.getInputName(this.props.path));

	},
	componentWillUnmount () {
		// todo: is there a lighter-weight way to remove the cm instance?
		if (this.codeMirror) {
			this.codeMirror.toTextArea();
		}
	},

	focus () {
		if (this.codeMirror) {
			this.codeMirror.focus();
		}
	},
	focusChanged (focused) {
		this.setState({
			isFocused: focused,
		});
	},
	codemirrorValueChanged (doc, change) {
		var newValue = doc.getValue();
		this.props.form.setFieldsValue({
			[this.getInputName(this.props.path)]:newValue ? newValue : ""
		})

	},
	renderCodemirror () {
		const className = classnames('CodeMirror-container', {
			'is-focused': this.state.isFocused && this.shouldRenderField(),
		});
		const { getFieldDecorator } = this.props.form;
		const name = this.getInputName(this.props.path);

		const hiddenInput = (
				<Input disabled type="hidden" name={name} />
		)
		return (
			<div className={className}>
				{hiddenInput}
				<Input.TextArea
					autoComplete="off"
					ref="codemirror"
					defaultValue={this.props.value}
				/>
			</div>
		);
	},
	renderValue () {
		return this.renderCodemirror();
	},
	renderField () {
		return this.renderCodemirror();
	},
});
