import React, { PropTypes } from 'react';
import { findDOMNode } from 'react-dom';
import {connect} from 'react-redux';
import moment from 'moment';
import _ from 'lodash';
import {
	DatePicker,
	Input,
	Select,
	Row,
	Col,
	Form,
} from 'antdlib';
import reduxMng from 'kadm-redux-mng';
var store = reduxMng.getStore();
import SegmentedControl from '../../components/SegmentedControl';

const INVERTED_OPTIONS = [
	{ label: '匹配', value: false },
	{ label: '不匹配', value: true },
];

const MODE_OPTIONS = [
	{ label: '当天', value: 'on' },
	{ label: '日期之后', value: 'after' },
	{ label: '日期之前', value: 'before' },
	{ label: '日期区间', value: 'between' },
];

const DayPickerIndicator = ({ activeInputField }) => {
	const style = activeInputField === 'before' ? { left: '11rem' } : null;

	return (
		<span className="DayPicker-Indicator" style={style}>
			<span className="DayPicker-Indicator__border" />
			<span className="DayPicker-Indicator__bg" />
		</span>
	);
};
moment.locale('zh-cn');
function getDefaultValue () {
	return {
		mode: MODE_OPTIONS[0].value,
		inverted: INVERTED_OPTIONS[0].value,
		value: moment(0, 'HH').format(),
		before: moment(0, 'HH').format(),
		after: moment(0, 'HH').format(),
	};
}

var DateFilter = React.createClass({
	displayName: 'DateFilter',
	propTypes: {
		filter: PropTypes.shape({
			mode: PropTypes.oneOf(MODE_OPTIONS.map(i => i.value)),
			inverted: PropTypes.boolean,
		}),
	},
	statics: {
		getDefaultValue: getDefaultValue,
	},
	getDefaultProps () {
		return {
			format: 'YYYY-MM-DD',
			filter: getDefaultValue(),
			value: moment().startOf('day').toDate(),
		};
	},
	getInitialState () {
		return {
			activeInputField: 'after',
			month: new Date(), // The month to display in the calendar
		};
	},
	componentDidMount () {
		// focus the text input
		// if (this.props.filter.mode === 'between') {
		// 	findDOMNode(this.refs[this.state.activeInputField]).focus();
		// } else {
		// 	this.refs.input.focus();
		// }
		
		
	},
	closeEventBlocker(e){
		console.log(e);
		e.stopImmediatePropagation();
		e.stopPropagation();
	},
	onOpenChange(open){
		var {actions} = require('kadm-component');
		var {
			lockPopover 
		} = actions.lists;
		var _lockPopover = (lock) =>{
			if (store.getState().lists)
				store.dispatch(lockPopover(lock));
		}
		if (open){
			_lockPopover(true);
		} else {
			_lockPopover(false);
			// $('.page-content')[0].removeEventListener('click', this.closeEventBlocker);
		}

	},
	updateFilter (value) {
		this.props.onChange({ ...this.props.filter, ...value });
	},
	toggleInverted (value) {
		this.updateFilter({ inverted: value });
		// this.setFocus(this.props.filter.mode);
	},
	selectMode (mode) {
		this.updateFilter({ mode });
		// this.setFocus(mode);
	},
	// setFocus (mode) {
	// 	if (mode === 'between') {
	// 		setTimeout(() => {
	// 			findDOMNode(this.refs[this.state.activeInputField]).focus();
	// 		}, 50);
	// 	} else {
	// 		setTimeout(() => {
	// 			this.refs.input.focus();
	// 		}, 50);
	// 	}
	// },
	handleInputChange (e) {
		const { value } = e.target;
		let { month } = this.state;
		// Change the current month only if the value entered by the user is a valid
		// date, according to the `L` format
		if (moment(value, 'L', true).isValid()) {
			month = moment(value, 'L').toDate();
		}
		this.updateFilter({ value: value });
		this.setState({ month }, this.showCurrentDate);
	},
	setActiveField (field) {
		this.setState({
			activeInputField: field,
		});
	},
	switchBetweenActiveInputFields (mobj, daystr, modifiers) {
		if (modifiers && modifiers.disabled) return;

		var send = {
			after:daystr[0],
			before:daystr[1],
		}
		this.updateFilter(send);
	},
	selectDay (mobj, daystr, modifiers) {
		if (modifiers && modifiers.disabled) return;
		this.updateFilter({ value: daystr });
	},
	showCurrentDate () {
		this.refs.daypicker.showMonth(this.state.month);
	},
	renderToggle () {
		const { filter } = this.props;
		return (
			<Form.Item>
				<SegmentedControl
					options={INVERTED_OPTIONS}
					defaultValue={filter.inverted}
					onChange={this.toggleInverted}
				/>
			</Form.Item>
		);
	},
	renderControls () {
		let controls;
		const { activeInputField } = this.state;
		const { field, filter } = this.props;
		const mode = MODE_OPTIONS.filter(i => i.value === filter.mode)[0];
		const placeholder = field.label + ' is ' + mode.label.toLowerCase() + '...';

		// DayPicker Modifiers - Selected Day
		let modifiers = filter.mode === 'between' ? {
			selected: (day) => moment(filter[activeInputField]).isSame(day),
		} : {
			selected: (day) => moment(filter.value).isSame(day),
		};

		let DateNode = mode.value === 'between' ? DatePicker.RangePicker : DatePicker;
		let dvalue = mode.value === 'between' ? [moment(filter.after), moment(filter.before)] : moment(filter.value);
		let handler = mode.value === 'between' ? this.switchBetweenActiveInputFields : this.selectDay;
		controls = (
			<Form.Item>
				<DateNode
					ref="daypicker"
					defaultValue={dvalue}
					format={this.props.format}
					modifiers={modifiers}
					onOpenChange={this.onOpenChange}
					onChange={handler}
					style={{
						width:'100%',
					}}
					
				/>
			</Form.Item>
		);
		

		return controls;
	},
	updateHeight(){
		var mainH = findDOMNode(this.refs.main).offsetHeight;
		this.props.onHeightChange(mainH + 341);
	},
	render () {
		const { filter } = this.props;
		const mode = MODE_OPTIONS.filter(i => i.value === filter.mode)[0];
		var options = _.map(MODE_OPTIONS, (o) => {
			return <Select.Option {...{
				key:o.value,
				value:o.value,
			}}>
			{o.label}
			</Select.Option>
		})
		return (
			<div ref='main'>
				{this.renderToggle()}
				<Form.Item>
					<Select
						onChange={this.selectMode}
						defaultValue={mode.value}
						getPopupContainer={() => findDOMNode(this.refs.main)}
					>
						{options}
					</Select>
				</Form.Item>
				
				{this.renderControls()}
			</div>
		);
	},
});

module.exports = DateFilter;