
import Field from '../Field';
import moment from 'moment';
import React from 'react';
import {DatePicker, Tag} from 'antdlib';
module.exports = Field.create({

	displayName: 'DatetimeField',
	statics: {
		type: 'Datetime',
		showTime:true,
	},

	focusTargetRef: 'dateInput',

	// parse formats (duplicated from lib/fieldTypes/datetime.js)
	parseFormats: ['YYYY-MM-DD', 'YYYY-MM-DD HH:mm:ss',  'YYYY-MM-DD HH:mm'],

	getInitialState () {
		return {
			value: this.moment(this.props.value).format(this.props.formatString)
			// dateValue: this.props.value && this.moment(this.props.value).format(this.dateInputFormat),
			// timeValue: this.props.value && this.moment(this.props.value).format(this.timeInputFormat),
			// tzOffsetValue: this.props.value ? this.moment(this.props.value).format(this.tzOffsetInputFormat) : this.moment().format(this.tzOffsetInputFormat),
		};
	},

	getDefaultProps () {
		return {
			formatString: 'YYYY-MM-DD HH:mm:ss',
		};
	},

	getInitValue(){
		const {  value,   defaultValue} = this.props;
		return moment(value || defaultValue)
	},
	moment () {
		if (this.props.isUTC) return moment.utc.apply(moment, arguments);
		else return moment.apply(undefined, arguments);
	},

	// TODO: Move isValid() so we can share with server-side code
	isValid (value) {
		return this.moment(value, this.parseFormats).isValid();
	},

	// TODO: Move format() so we can share with server-side code
	format (value, format) {
		format = format || this.props.formatString;
		return value ? this.moment(value).format(format) : '';
	},

	// handleChange (dateValue, timeValue, tzOffsetValue) {
	// 	var value = dateValue + ' ' + timeValue;
	// 	var datetimeFormat = this.dateInputFormat + ' ' + this.timeInputFormat;
    //
	// 	// if the change included a timezone offset, include that in the calculation (so NOW works correctly during DST changes)
	// 	if (typeof tzOffsetValue !== 'undefined') {
	// 		value += ' ' + tzOffsetValue;
	// 		datetimeFormat += ' ' + this.tzOffsetInputFormat;
	// 	}
	// 	// if not, calculate the timezone offset based on the date (respect different DST values)
	// 	else {
	// 		this.setState({ tzOffsetValue: this.moment(value, datetimeFormat).format(this.tzOffsetInputFormat) });
	// 	}
    //
	// 	this.props.onChange({
	// 		path: this.props.path,
	// 		value: this.isValid(value) ? this.moment(value, datetimeFormat).toISOString() : null,
	// 	});
	// },

	// dateChanged ({ value }) {
	// 	this.setState({ dateValue: value });
	// 	this.handleChange(value, this.state.timeValue);
	// },
    //
	// timeChanged (evt) {
	// 	this.setState({ timeValue: evt.target.value });
	// 	this.handleChange(this.state.dateValue, evt.target.value);
	// },

	// setNow () {
	// 	var dateValue = this.moment().format(this.dateInputFormat);
	// 	var timeValue = this.moment().format(this.timeInputFormat);
	// 	var tzOffsetValue = this.moment().format(this.tzOffsetInputFormat);
	// 	this.setState({
	// 		dateValue: dateValue,
	// 		timeValue: timeValue,
	// 		tzOffsetValue: tzOffsetValue,
	// 	});
	// 	this.handleChange(dateValue, timeValue, tzOffsetValue);
	// },

	// renderNote () {
	// 	if (!this.props.note) return null;
	// 	return <FormNote note={this.props.note} />;
	// },

	renderValue(){
		return  <Tag>{this.format(this.props.value, this.props.formatString)}</Tag>;
	},
	renderField () {
		return (

			<DatePicker
				size='large'
				showTime
				format={this.props.formatString}
				placeholder="选择时间"

			/>
		)
	},

	// renderUI () {
	// 	var input;
	// 	if (this.shouldRenderField()) {
	// 		input = (
	// 			<InputGroup>
	// 				<InputGroup.Section grow>
	// 					<DateInput ref="dateInput" name={this.getInputName(this.props.paths.date)} value={this.state.dateValue} format={this.dateInputFormat} onChange={this.dateChanged} />
	// 				</InputGroup.Section>
	// 				<InputGroup.Section grow>
	// 					<FormInput name={this.getInputName(this.props.paths.time)} value={this.state.timeValue} placeholder="HH:MM:SS am/pm" onChange={this.timeChanged} autoComplete="off" />
	// 				</InputGroup.Section>
	// 				<InputGroup.Section>
	// 					<Button onClick={this.setNow}>Now</Button>
	// 				</InputGroup.Section>
	// 				<input type="hidden" name={this.getInputName(this.props.paths.tzOffset)} value={this.state.tzOffsetValue} />
	// 			</InputGroup>
	// 		);
	// 	} else {
	// 		input = <FormInput noedit>{this.format(this.props.value, this.props.formatString)}</FormInput>;
	// 	}
	// 	return (
	// 		<FormField label={this.props.label} className="field-type-datetime" htmlFor={this.getInputName(this.props.path)}>
	// 			{input}
	// 			{this.renderNote()}
	// 		</FormField>
	// 	);
	// },
});
