/**
TODO:
- Format size of stored file (if present) using bytes package?
- Display file type icon? (see LocalFileField)
*/

import Field from '../Field';
import React, { PropTypes } from 'react';
import _ from 'lodash';
import {
	Input,
	Upload,
	Icon
} from 'antdlib';
const Dragger = Upload.Dragger;


/**
 *
 * props use in model
 *
 * listType: "text", "picture", "picture-card", //see that antd design in upload
 *
 *
 */


module.exports = Field.create({

	propTypes: {
		listType: React.PropTypes.string,
	},
	getInitialProps(){
		return {
			listType:"text",
		}
	},
	handleBeforeUpload(file, fileList){
		var fileListNow = this.props.form.getFieldValue(this.getInputName(this.props.path)) || [];
		if (fileList.length + fileListNow.length > this.props.limit && this.props.limit > 0) return false;
		return true;
	},
	handleInputChanged(info){
		/**
		 * upload response text
		 *
		 * error
		 :
		 undefined
		 lastModified
		 :
		 1509353311000
		 lastModifiedDate
		 :
		 Mon Oct 30 2017 16:48:31 GMT+0800 (CST) {}
		 name
		 :
		 "车联网登录end.jpg"
		 originFileObj
		 :
		 File {uid: "rc-upload-1510631707626-2", name: "车联网登录end.jpg", lastModified: 1509353311000, lastModifiedDate: Mon Oct 30 2017 16:48:31 GMT+0800 (CST), webkitRelativePath: "", …}
		 percent
		 :
		 100
		 response
		 :
		 {status: 1, info: {…}}
		 size
		 :
		 102531
		 status
		 :
		 "done"
		 type
		 :
		 "image/jpeg"
		 uid
		 :
		 "rc-upload-1510631707626-2"
		 *
		 *
		 */
		const status = info.file.status;
		switch (status){
			case "done":
				var fileList = this.props.form.getFieldValue(this.getInputName(this.props.path)) || [];
				let value = _.assign({}, info.file.response.info.result, {
					uid:info.file.uid,
					lastModified:info.file.lastModified
				});
				fileList.push(value);
				this.props.form.setFieldsValue({
					[this.getInputName(this.props.path)]:fileList,
				});
				break;
			case "removed":
				var fileList = this.props.form.getFieldValue(this.getInputName(this.props.path)) || [];
				_.remove(fileList, (o) => {
					return o.uid == info.file.uid;
				})
				this.props.form.setFieldsValue({
					[this.getInputName(this.props.path)]:fileList,
				});
				break;
			default:
				break;
		}

	},
	format(){
		var value = this.props.form.getFieldValue(this.getInputName(this.props.path));
		return JSON.stringify(value);
	},
	shouldUseFieldDecorator(){
		return false;
	},
	renderField(){



		const {   path} = this.props;
		const { getFieldDecorator } = this.props.form;

		let node = getFieldDecorator(
			path,
			{
				rules: this.getValidateRules(),
				initialValue:this.getInitValue(),
			}
		)(
				<Input
					{...{

						type:"hidden",
						name:this.getInputName(this.props.path)
					}}
				/>
		)

		var fileList = this.props.form.getFieldValue(this.getInputName(this.props.path));
		var value = _.map(fileList, function(o){
			return {
				// uid: -1,
				// name: 'xxx.png',
				// status: 'done',
				// url: 'http://www.baidu.com/xxx.png',
				uid:o.uid,
				name:o.originalname,
				mimetype:o.mimetype,
				status: 'done',
				url:o.url,
			}
		})
		var uploadProps = {
			name: 'file',
			multiple: true,
			showUploadList: true,
			action: '/admin/api2/upload/',// + this.props.belongModel,
			defaultFileList:value,
			headers:{
				"x-csrf-token":KAdm.csrf.header["x-csrf-token"],
			},
			data:{
				uploadPath:this.props.uploadPath || "",
				schema:JSON.stringify(this.props.schema)
			},
			listType:this.props.listType,
			onChange:this.handleInputChanged.bind(this),
			beforeUpload:this.handleBeforeUpload.bind(this),
		}
		return (
			<div style={{ marginTop: 16 }}>
				<Dragger {...uploadProps}>
					<p className="ant-upload-drag-icon">
						<Icon type="inbox" />
					</p>
					<p className="ant-upload-text">点击或拖动文件到这里进行上传</p>
					<p className="ant-upload-hint">支持多个文件上传. </p>
				</Dragger>
				{node}
			</div>
		);
	}
})

