import React, { Component, PropTypes } from 'react';
import vkey from 'vkey';
import {
	message
} from 'antdlib';
// import { Button, FormField, FormNote, SegmentedControl } from 'elemental';
// var {
// 	shared:{
// 		PopoutList
// 	}
// } = require('kadm-component');

import PopoutList from '../../../admin/client/App/shared/Popout/PopoutList';


import {
	Button,
	Form,
} from 'antdlib';

import SegmentedControl from '../../components/SegmentedControl';

import bindFunctions from '../../utils/bindFunctions';

const INVERTED_OPTIONS = [
	{ label: '匹配', value: false },
	{ label: '不匹配', value: true },
];

function getDefaultValue () {
	return {
		inverted: INVERTED_OPTIONS[0].value,
		value: [],
	};
}

class FilterOption extends Component {
	constructor () {
		super();

		bindFunctions.call(this, [
			'handleClick',
		]);
	}
	handleClick () {
		const { option, selected } = this.props;
		this.props.onClick(option, selected);
	}
	render () {
		const { option, selected } = this.props;
		return (
			<PopoutList.Item
				icon={selected ? 'check' : 'dash'}
				isSelected={selected}
				label={option.label}
				onClick={this.handleClick}
			/>
		);
	}
}

class SelectFilter extends Component {
	constructor () {
		super();

		bindFunctions.call(this, [
			'detectOS',
			'handleClick',
			'handleKeyDown',
			'handleKeyUp',
			'removeOption',
			'selectOption',
			'toggleAllOptions',
			'toggleInverted',
			'updateFilter',
		]);

		this.state = { metaDown: false };
	}
	componentDidMount () {
		this.detectOS();
		document.body.addEventListener('keydown', this.handleKeyDown, false);
		document.body.addEventListener('keyup', this.handleKeyUp, false);
	}
	componentWillUnmount () {
		document.body.removeEventListener('keydown', this.handleKeyDown);
		document.body.removeEventListener('keyup', this.handleKeyUp);
	}

	// ==============================
	// METHODS
	// ==============================

	// TODO this should probably be moved to the main App component and stored
	// in context for other components to subscribe to when required
	detectOS () {
		let osName = 'Unknown OS';

		if (navigator.appVersion.includes('Win')) osName = 'Windows';
		if (navigator.appVersion.includes('Mac')) osName = 'MacOS';
		if (navigator.appVersion.includes('X11')) osName = 'UNIX';
		if (navigator.appVersion.includes('Linux')) osName = 'Linux';

		this.setState({ osName });
	}
	handleKeyDown (e) {
		if (vkey[e.keyCode] !== '<meta>') return;

		this.setState({ metaDown: true });
	}
	handleKeyUp (e) {
		if (vkey[e.keyCode] !== '<meta>') return;

		this.setState({ metaDown: false });
	}

	toggleInverted (inverted) {
		this.updateFilter({ inverted });
	}
	toggleAllOptions () {
		const { field, filter } = this.props;

		if (filter.value.length < field.ops.length) {
			this.updateFilter({ value: field.ops.map(i => i.value) });
		} else {
			this.updateFilter({ value: [] });
		}
	}
	selectOption (option) {
		const value = this.state.metaDown
			? this.props.filter.value.concat(option.value)
			: [option.value];
		const label = this.state.metaDown
			? this.props.filter.label.concat(option.label)
			: [option.label];

		this.updateFilter({ value,label });
	}
	removeOption (option) {
		const value = this.state.metaDown
			? this.props.filter.value.filter(i => i !== option.value)
			: [option.value];
		const label = this.state.metaDown
			? this.props.filter.label.filter(i => i !== option.label)
			: [option.label];

		this.updateFilter({ value, label });
	}
	handleClick (option, selected) {
		selected ? this.removeOption(option) : this.selectOption(option);
	}
	updateFilter (value) {
		this.props.onChange({ ...this.props.filter, ...value });
	}

	// ==============================
	// RENDERERS
	// ==============================

	renderOptions () {
		return this.props.field.ops.map((option, i) => {
			const selected = this.props.filter.value.indexOf(option.value) > -1;
			return (
				<FilterOption
					key={`item-${i}-${option.value}`}
					option={option}
					selected={selected}
					onClick={this.handleClick}
				/>
			);
		});
	}
	render () {
		const { field, filter } = this.props;
		const indeterminate = filter.value.length < field.ops.length;

		const metaKeyLabel = this.state.osName === 'MacOS'
			? 'cmd'
			: 'ctrl';

		const noteStr = <div>
				Hold <kbd>{metaKeyLabel}</kbd> to select multiple options
			</div>

		const fieldStyles = {
			alignItems: 'center',
			borderBottom: '1px dashed rgba(0,0,0,0.1)',
			display: 'flex',
			justifyContent: 'space-between',
			marginBottom: '1em',
			paddingBottom: '1em',
		};

		return (
			<div>
				<Form.Item>
					<SegmentedControl
						onChange={this.toggleInverted}
						options={INVERTED_OPTIONS}
						defaultValue={filter.inverted}
					/>
				</Form.Item>
				<Form.Item style={fieldStyles} extra={noteStr}>
					<Button onClick={this.toggleAllOptions} style={{ padding: 0, width: '100%' }}>
						{indeterminate ? 'All' : 'None'}
					</Button>
				</Form.Item>
				{this.renderOptions()}
			</div>
		);
	}
};


SelectFilter.propTypes = {
	field: PropTypes.object,
	filter: PropTypes.shape({
		inverted: PropTypes.boolean,
		value: PropTypes.array,
	}),
};
SelectFilter.getDefaultValue = getDefaultValue;
SelectFilter.defaultProps = {
	filter: getDefaultValue(),
};

module.exports = SelectFilter;
