import Field from '../Field';
import React, { PropTypes } from 'react';
import {
	Input,
	Form,
	Row,
	Col,
} from 'antdlib'

//import { FormField, FormInput, FormRow } from 'elemental';

const NAME_SHAPE = {
	first: PropTypes.string,
	last: PropTypes.string,
};

module.exports = Field.create({
	displayName: 'NameField',
	statics: {
		type: 'Name',
		getDefaultValue: () => ({
			first: '',
			last: '',
		}),
	},
	propTypes: {
		onChange: PropTypes.func.isRequired,
		path: PropTypes.string.isRequired,
		paths: PropTypes.shape(NAME_SHAPE).isRequired,
		value: PropTypes.shape(NAME_SHAPE).isRequired,
	},
	shouldUseFieldDecorator(){
		return false;
	},
	//getInputValue(path){
	//	const name = this.getInputName(path);
	//	const value = this.props.form.getFieldValue(name);
	//	return typeof value == "object" ? value : JSON.stringify(value);
	//},
	valueChanged: function (which, event) {

		const name = this.getInputName(this.props.path);
		const value = this.props.form.getFieldValue(name);
		const newValue = {
			...value,
			[which]: event.target.value || "",
		}


		this.props.form.setFieldsValue({
			[name]:newValue
		});

		//this.refs.result.value = newValue;
		//onChange({
		//	path,
		//	value: {
		//		...value,
		//		[which]: event.target.value,
		//	},
		//});
	},
	format(){
		var value = this.props.form.getFieldValue(this.getInputName(this.props.path));
		value.first = value.first || "";
		value.last = value.last || "";
		return JSON.stringify(value);
	},
	changeFirst: function (event) {
		return this.valueChanged('first', event);
	},
	changeLast: function (event) {
		return this.valueChanged('last', event);
	},
	renderValue () {
		const inputStyle = { width: '100%' };
		const { value = {} } = this.props;

		return (
			<Row>
				<Col span="12">
					<Input disabled style={inputStyle}>
						{value.first}
					</Input>
				</Col>
				<Col span="12">
					<Input disabled style={inputStyle}>
						{value.last}
					</Input>
				</Col>
			</Row>
		);
	},
	renderField () {

		const inputStyle = { width: '90%' };
		const {paths} = this.props;
		const name = this.getInputName(this.props.path);


		const { getFieldDecorator } = this.props.form;
		const hiddenInput = (


			getFieldDecorator(
				name,
				{
					rules: this.getValidateRules(),
					initialValue:this.getInitValue(),
				}
			)(
				<Input disabled type="hidden" name={name}  />
			)
		);
		var value = this.props.form.getFieldValue(name);
		if (!value) {
			value = {
				first:"",
				last:"",
			}
			this.props.form.setFieldsValue({
				[name]:value,
			})
		}
		return (
			<Row>
				{hiddenInput}
				<Col span="12">
					<Input
						style={inputStyle}
						name={this.getInputName(paths.first)}
						onChange={this.changeFirst.bind(this)}
						value={value.first}
						size={this.fieldSize}
					/>
				</Col>
				<Col span="12">
					<Input
						style={inputStyle}
						name={this.getInputName(paths.last)}
						onChange={this.changeLast.bind(this)}
						value={value.last}
						size={this.fieldSize}
					/>


				</Col>
			</Row>
			//<FormRow>
			//	<FormField width="one-half">
			//		<FormInput
			//			autoFocus={autoFocus}
			//			autoComplete="off"
			//			name={this.getInputName(paths.first)}
			//			onChange={this.changeFirst}
			//			placeholder="First name"
			//			value={value.first}
			//		/>
			//	</FormField>
			//	<FormField width="one-half">
			//		<FormInput
			//			autoComplete="off"
			//			name={this.getInputName(paths.last)}
			//			onChange={this.changeLast}
			//			placeholder="Last name"
			//			value={value.last}
			//		/>
			//	</FormField>
			//</FormRow>
		);
	},
});
