import React from 'react';
import { findDOMNode } from 'react-dom';
import SegmentedControl from '../../components/SegmentedControl';
import {
	Form,
	InputNumber,
	Row,
	Col,
	Select,
} from 'antdlib';
// import { FormField, FormInput, FormRow, FormSelect } from 'elemental';

const INVERTED_OPTIONS = [
	{ label: '匹配', value: false },
	{ label: '不匹配', value: true },
];

const MODE_OPTIONS = [
	{ label: '等于', value: 'equals' },
	{ label: '大于', value: 'gt' },
	{ label: '小于', value: 'lt' },
	{ label: '区间', value: 'between' },
];

function getDefaultValue () {
	return {
		mode: MODE_OPTIONS[0].value,
		inverted: INVERTED_OPTIONS[0].value,
		value: 0,
	};
}

var NumberFilter = React.createClass({
	statics: {
		getDefaultValue: getDefaultValue,
	},
	getDefaultProps () {
		return {
			filter: getDefaultValue(),
		};
	},

	componentDidMount () {
		// focus the text input
		// findDOMNode(this.refs.focusTarget).focus();
	},
	selectMode (mode) {
		this.updateFilter({ mode });
		// findDOMNode(this.refs.focusTarget).focus();
	},
	handleChangeBuilder (type) {
		const self = this;
		return function handleChange (value) {
			const { filter, onChange } = self.props;

			switch (type) {
				case 'minValue':
					onChange({
						inverted:filter.inverted,
						mode: filter.mode,
						value: {
							min: value,
							max: filter.value.max,
						},
					});
					break;
				case 'maxValue':
					onChange({
						inverted:filter.inverted,
						mode: filter.mode,
						value: {
							min: filter.value.min,
							max: value,
						},
					});
					break;
				case 'value':
					onChange({
						inverted:filter.inverted,
						mode: filter.mode,
						value: value,
					});
			}
		};
	},
	// Update the props with this.props.onChange
	updateFilter (changedProp) {
		this.props.onChange({ ...this.props.filter, ...changedProp });
	},
	toggleInverted (inverted) {
		this.updateFilter({ inverted });
		// findDOMNode(this.refs.focusTarget).focus();
	},
	// Update the filter mode
	selectMode (mode) {
		this.updateFilter({ mode });

		// focus on next tick
		setTimeout(() => {
			findDOMNode(this.refs.focusTarget).focus();
		}, 0);
	},

	renderControls (mode) {
		const { filter } = this.props;
		let controls;
		const { field } = this.props;
		const placeholder = field.label + ' is ' + mode.label.toLowerCase() + '...';

		if (mode.value === 'between') {
			controls = (
				<Row gutter='32'>
					<Col span='12'>
						<Form.Item width="one-half" style={{ marginBottom: 0 }}>
							<InputNumber
								onChange={this.handleChangeBuilder('minValue')}
								placeholder="最小值"
								ref="focusTarget"
								type="number"
								defaultValue={filter.value.min}
								style={{
									width:'100%',
								}}
							/>
						</Form.Item>
					</Col>
					<Col span='12'>
						<Form.Item width="one-half" style={{ marginBottom: 0 }}>
							<InputNumber
								onChange={this.handleChangeBuilder('maxValue')}
								placeholder="最大值"
								type="number"
								defaultValue={filter.value.max}
								style={{
									width:'100%',
								}}
							/>
						</Form.Item>
					</Col>
					
				</Row>
			);
		} else {
			controls = (
				<Form.Item>
					<InputNumber
						onChange={this.handleChangeBuilder('value')}
						placeholder={placeholder}
						ref="focusTarget"
						type="number"
						defaultValue={filter.value}
						style={{
							width:'100%',
						}}
					/>
				</Form.Item>
			);
		}

		return controls;
	},

	render () {
		const { filter } = this.props;
		const mode = MODE_OPTIONS.filter(i => i.value === filter.mode)[0];
		var options = _.map(MODE_OPTIONS, (o) => {
			return <Select.Option {...{
				key:o.value,
				value:o.value,
			}}>
			{o.label}
			</Select.Option>
		})
		return (
			<div  ref='main'>
				<Form.Item>
					<SegmentedControl options={INVERTED_OPTIONS} defaultValue={filter.inverted} onChange={this.toggleInverted} />
				</Form.Item>
				<Form.Item>
					<Select
						onChange={this.selectMode}
						defaultValue={mode.value}
						getPopupContainer={() => findDOMNode(this.refs.main)}
					>
						{options}
					</Select>
				</Form.Item>
				{this.renderControls(mode)}
			</div>
		);
	},

});

module.exports = NumberFilter;
