import Field from '../Field';
import React from 'react';
import _ from 'lodash';
import URL from 'url';

/**
 * TODO:
 * - Custom path support
 */
import {
	Select,
	Input,
	Row,
	Col,
	AutoComplete
} from "antdlib";
const Option = Select.Option;

const AutoCompleteOption = AutoComplete.Option;

module.exports = Field.create({

	displayName: 'QRCodeField',
	statics: {
		type: 'QRCode',
	},
	getInitialState () {
		return {
			protocol:'http:',
		};
	},
	shouldUseFieldDecorator(){
		return false;
	},
	format(){
		var value = this.props.form.getFieldValue(this.getInputName(this.props.path));
		value.type = value.type || 0;
		value.value = value.value||'';
		return JSON.stringify(value);
	},
	typeValueChanged (newValue) {
		const name = this.getInputName(this.props.path);
		let content = this.props.form.getFieldValue(name);
		
		this.props.form.setFieldsValue({
			[name]:_.assign(content, {type:newValue}),
		})
	},
	protocolValueChanged(newValue){
		this.setState({
			protocol:newValue,
		})
		const name = this.getInputName(this.props.path);
		let content = this.props.form.getFieldValue(name);
		let value = newValue + "//" + this.refs.input.input.value
		this.props.form.setFieldsValue({
			[name]:_.assign(content, {value}),
		})
	},
	handleTextChange(e){
		const name = this.getInputName(this.props.path);
		let content = this.props.form.getFieldValue(name);
		let value = e.currentTarget.value
		this.props.form.setFieldsValue({
			[name]:_.assign(content, {value}),
		})
	},
	handleWebsiteChange(e){
		const name = this.getInputName(this.props.path);
		let content = this.props.form.getFieldValue(name);
		let value = this.state.protocol + "//" + e.currentTarget.value
		this.props.form.setFieldsValue({
			[name]:_.assign(content, {value}),
		})
	},
	renderValue () {
		var selected = this.props.ops.find(option => option.value === this.props.value);
		return <Input disabled>{selected ? selected.label : null}</Input>;
	},
	renderOptions(){
		var opts = [
			{value:0,label:'URL'},
			{value:1,label:'Text'},
		];
		return _.map(opts, (o) => {
			return <Select.Option key={o.value} value={o.value}>{o.label}</Select.Option>;
		})
	},

	renderField () {
		// TODO: This should be natively handled by the Select component
		// var ops = (this.props.numeric) ? this.props.ops.map(function (i) { return { label: i.label, value: i.value}; }) : this.props.ops;
		

		const name = this.getInputName(this.props.path);

		const { getFieldDecorator } = this.props.form;
		const hiddenInput = (


			getFieldDecorator(
				name,
				{
					rules: this.getValidateRules(),
					initialValue:this.getInitValue(),
				}
			)(
				<Input disabled type="hidden" name={name}  />
			)
		);


		let content = this.props.form.getFieldValue(name);

		let secondNode;
		
		if (content.type == 1){
			secondNode = <Input.TextArea
				autosize={{ minRows: 2, maxRows: 6 }}
				ref='input'
				onChange={this.handleTextChange.bind(this)}
				defaultValue={content.value}
			/>
		} else if (content.type == 0) {
			let url = URL.parse(content.value);

			const addonBefore = <Select
				defaultValue={url.protocol || this.state.protocol} 
				style={{ width: 90 }}
				ref='protocol'
				onChange={this.protocolValueChanged.bind(this)}
				>
				<Option value="http:">http://</Option>
				<Option value="https:">https://</Option>
			</Select>


			secondNode = <Input
				size="large"
				ref='input'
				addonBefore={addonBefore}
				onChange={this.handleWebsiteChange.bind(this)}
				placeholder="website url"
				defaultValue={url.host + url.path}
			/>
		}

		

		const { autoFocus, inputProps, path } = this.props;
		return (
			<div>
				{hiddenInput}
				<Row>
					<Col span="6">
						<Select {...{
							autoFocus,
							size:'large',
							onChange: this.typeValueChanged.bind(this),
							defaultValue:content.type,
						}}>
							{this.renderOptions()}
						</Select>
					</Col>
				</Row>
				<Row style={{
					marginTop:'15px',
				}}>
					{secondNode}
				</Row>
				
			</div>
			
			
		);
	},

});
