var _ = require('lodash');
var FieldType = require('../Type');
var util = require('util');
var utils = require('keystone-utils');

/**
 * Select FieldType Constructor
 * @extends Field
 * @api public
 */
function qrcode (list, path, options) {
	this.ui = options.ui || 'qrcode';
	this.numeric = options.numeric ? true : false;
	this._underscoreMethods = ['format'];
	options.default = { type: 0, value: '' };
	// if (typeof options.options === 'string') {
	// 	options.options = options.options.split(',');
	// }
	// if (!Array.isArray(options.options)) {
	// 	throw new Error('Select fields require an options array.');
	// }
	// this.ops = options.options.map(function (i) {
	// 	var op = typeof i === 'string' ? { value: i.trim(), label: utils.keyToLabel(i) } : i;
	// 	if (!_.isObject(op)) {
	// 		op = { label: '' + i, value: '' + i };
	// 	}
	// 	if (options.numeric && !_.isNumber(op.value)) {
	// 		op.value = Number(op.value);
	// 	}
	// 	return op;
	// });
	// // undefined options.emptyOption defaults to true
	// if (options.emptyOption === undefined) {
	// 	options.emptyOption = true;
	// }
	// // ensure this.emptyOption is a boolean
	// this.emptyOption = !!options.emptyOption;
	// // cached maps for options, labels and values
	// this.map = utils.optionsMap(this.ops);
	// this.labels = utils.optionsMap(this.ops, 'label');
	// this.values = _.map(this.ops, 'value');
	qrcode.super_.call(this, list, path, options);
}
qrcode.properName = 'QRCode';
util.inherits(qrcode, FieldType);

/**
 * Registers the field on the List's Mongoose Schema.
 *
 * Adds a virtual for accessing the label of the selected value,
 * and statics to the Schema for converting a value to a label,
 * and retrieving all of the defined options.
 */
qrcode.prototype.addToSchema = function (schema) {
	var field = this;
	var ops = (this._nativeType) ? _.defaults({ type: this._nativeType }, this.options) : this.options;
	schema.path(this.path, Object);
	this.bindUnderscoreMethods();
};

/**
 * Returns a key value from the selected option
 */
// select.prototype.pluck = function (item, property, _default) {
// 	var option = item.get(this.paths.data);
// 	return (option) ? option[property] : _default;
// };

/**
 * Retrieves a shallow clone of the options array
 */
// select.prototype.cloneOps = function () {
// 	return _.map(this.ops, _.clone);
// };

/**
 * Retrieves a shallow clone of the options map
 */
// select.prototype.cloneMap = function () {
// 	return utils.optionsMap(this.ops, true);
// };

/**
 * Add filters to a query
 */
// select.prototype.addFilterToQuery = function (filter) {
// 	var query = {};
// 	if (!Array.isArray(filter.value)) {
// 		if (filter.value) {
// 			filter.value = [filter.value];
// 		} else {
// 			filter.value = [];
// 		}
// 	}
// 	if (filter.value.length > 1) {
// 		query[this.path] = (filter.inverted) ? { $nin: filter.value } : { $in: filter.value };
// 	} else if (filter.value.length === 1) {
// 		query[this.path] = (filter.inverted) ? { $ne: filter.value[0] } : filter.value[0];
// 	} else {
// 		query[this.path] = (filter.inverted) ? { $nin: ['', null] } : { $in: ['', null] };
// 	}
// 	return query;
// };

/**
 * Asynchronously confirms that the provided value is valid
 */
qrcode.prototype.validateInput = function (data, callback) {
	var value = this.getInputFromData(data);
	
	var result = value === undefined || value === null || value === '' || typeof value === 'object' ? true : false;
	utils.defer(callback, result);
};

/**
 * Asynchronously confirms that the provided value is present
 */
qrcode.prototype.validateRequiredInput = function (item, data, callback) {
	var value = this.getInputFromData(data);
	var result = false;
	if (value === undefined) {
		if (item.get(this.path)) {
			result = true;
		}
	} else if (value) {
		if (value !== '') {
			// This is already checkind in validateInput, but it doesn't hurt
			// to check again for security
			if (value in this.map) {
				result = true;
			}
		}
	}
	utils.defer(callback, result);
};

/**
 * Validates that a valid option has been provided in a data object
 *
 * Deprecated
 */
qrcode.prototype.inputIsValid = function (data, required, item) {
	if (data[this.path]) {
		return (data[this.path] in this.map) ? true : false;
	} else {
		return (!required || (!(this.path in data) && item && item.get(this.path))) ? true : false;
	}
};

/**
 * Formats the field value
 */
// qrcode.prototype.format = function (item) {
// 	return this.labels[item.get(this.path)] || '';
// };

/**
 * Get the value from a data object; may be simple or a pair of fields
 */
qrcode.prototype.getInputFromData = function (data) {
	// this.getValueFromData throws an error if we pass name: null
	if (data[this.path] === null || data[this.path] === undefined) {
		return {type:0, value:""};
	}
	else if (typeof data[this.path] == "object"){
		return data[this.path];
	}
	return JSON.parse(data[this.path]);

};

qrcode.prototype.updateItem = function (item, data, callback) {
	var path = this.path;
	var value = this.getInputFromData(data);
	if (typeof value === 'object') {
		
		item.set(path, value);
	}
	process.nextTick(callback);
};

/* Export Field Type */
module.exports = qrcode;
