import async from 'async';
import Field from '../Field';
import { listsByKey } from '../../../admin/client/utils/lists';
import React from 'react';
import xhr from 'xhr';
// import { Button, InputGroup } from 'elemental';
import _ from 'lodash';
import {
	Select,
	Input,
	Tag
} from 'antdlib'
const Option = Select.Option;

function compareValues (current, next) {
	const currentLength = current ? current.length : 0;
	const nextLength = next ? next.length : 0;
	if (currentLength !== nextLength) return false;
	for (let i = 0; i < currentLength; i++) {
		if (current[i] !== next[i]) return false;
	}
	return true;
}

module.exports = Field.create({

	displayName: 'RelationshipField',
	statics: {
		type: 'Relationship',
	},

	getInitialState () {
		return {
			value: "",
			selectCValue:"",
			createIsOpen: false,
		};
	},

	componentDidMount () {
		this._itemsCache = {};
		this.loadOptions ("")
		this.loadValue(this.props.value);
	},

	componentWillReceiveProps (nextProps) {
		if (nextProps.value === this.props.value || nextProps.many && compareValues(this.props.value, nextProps.value)) return;
		this.loadValue(nextProps.value);
	},
	componentWillMount(){
		this.setState({
			selectCValue:this.props.value || this.props.defaultValue,
		})
	},
	getInitState(){
		var state = super.getInitState();
		return _.assign(state, {
			selectCValue: state.value
		})
	},
	shouldCollapse () {
		if (this.props.many) {
			// many:true relationships have an Array for a value
			return this.props.collapse && !this.props.value.length;
		}
		return this.props.collapse && !this.props.value;
	},
	buildFilters () {
		var filters = {};

		_.forEach(this.props.filters, (value, key) => {
			if (typeof value === 'string' && value[0] === ':') {
				var fieldName = value.slice(1);

				var val = this.props.values[fieldName];
				if (val) {
					filters[key] = val;
					return;
				}

				// check if filtering by id and item was already saved
				if (fieldName === ':_id' && Keystone.item) {
					filters[key] = Keystone.item.id;
					return;
				}
			} else {
				filters[key] = value;
			}
		}, this);

		var parts = [];

		_.forEach(filters, function (val, key) {
			parts.push('filters[' + key + '][value]=' + encodeURIComponent(val));
		});

		return parts.join('&');
	},

	cacheItem (item) {
		item.href = KAdm.adminCore + '/item/' + this.props.refList.path + '/' + item.id;
		this._itemsCache[item.id] = item;
	},

	loadValue (values) {
		if (!values) {
			return this.setState({
				loading: false,
				selectCValue:"",
			});
		};
		values = Array.isArray(values) ? values : values.split(',');
		const cachedValues = values.filter(i => i);
		if (cachedValues.length === values.length) {
			this.setState({
				loading: false,
				selectCValue:this.props.many ? cachedValues : cachedValues[0],
			});
			return;
		}
		this.setState({
			loading: true,
			selectCValue:"",
		});
		async.map(values, (value, done) => {
			xhr({
				url: KAdm.adminCore + '/api/' + this.props.refList.path + '/' + value + '?basic',
				responseType: 'json',
			}, (err, resp, data) => {
				if (err || !data) return done(err);
				this.cacheItem(data);
				done(err, data);
			});
		}, (err, expanded) => {
			if (!this.isMounted()) return;

			var idValue = _.map(expanded, o => o.id).filter(i => i)
			this.setState({
				loading: false,
				selectCValue:this.props.many ? idValue : idValue[0],
			});
		});
	},

	// NOTE: this seems like the wrong way to add options to the Select
	loadOptions (input) {
		// NOTE: this seems like the wrong way to add options to the Select

		const filters = this.buildFilters();
		xhr({
			url: KAdm.adminCore + '/api/' + this.props.refList.path + '?basic&search=' + input + '&' + filters,
			responseType: 'json',
		}, (err, resp, data) => {
			if (err) {
				console.error('Error loading items:', err);
				return callback(null, []);
			}
			data.results.forEach(this.cacheItem);
			this.setState({
				ops:data.results
			})
		});
	},


	openCreate () {
		this.setState({
			createIsOpen: true,
		});
	},

	closeCreate () {
		this.setState({
			createIsOpen: false,
		});
	},
	valueChanged(value){
		this.props.form.setFieldsValue({
			[this.getInputName(this.props.path)]:value ? value : ""
		});
		// this.setState({
		// 	selectCValue:value ? value : "",
		// })
	},
	onCreate (item) {
		this.cacheItem(item);
		if (Array.isArray(this.state.value)) {
			// For many relationships, append the new item to the end
			const values = this.state.value.map((item) => item.id);
			values.push(item.id);
			this.valueChanged(values.join(','));
		} else {
			this.valueChanged(item.id);
		}

		// NOTE: this seems like the wrong way to add options to the Select
		this.loadOptionsCallback(null, {
			complete: true,
			options: Object.keys(this._itemsCache).map((k) => this._itemsCache[k]),
		});
		this.closeCreate();
	},
	getInitValue(){
		const {  value,   defaultValue} = this.props;
		var v = value || defaultValue;
		if (Array.isArray(v)) return v;
		else if (typeof v == "string" && v.length == 0) v = [];
		return v;
	},
	format(){
		var value = this.props.form.getFieldValue(this.getInputName(this.props.path));
		return value.join(',')
	},
	renderSelect (noedit) {

		//var ops = (this.props.numeric) ? this.props.value.map(function (i) { return { label: i.label, value: String(i.value) }; }) : this.props.ops;
		const { autoFocus, inputProps, path, key } = this.props;
		let selectCValue = this.props.form.getFieldValue(this.getInputName(path));
		if (!noedit){
			var opsList = _.map(this.state.ops, (o) => {
				return <Option key={o.id} value={o.id}>{o.name}</Option>;
			});

			return (

				<Select {...{
					...inputProps,
					autoFocus,
					key,
					mode:this.props.many ? "multiple":null,
					disabled:noedit,
					defaultValue:this.props.value || this.props.defaultValue,
					value:selectCValue,
					size:'large',
					allowClear:true,
					onChange:this.valueChanged.bind(this),
				}}>
					{opsList}
				</Select>
			);
		} else {
			selectCValue = this.props.many ? this.state.selectCValue : [this.state.selectCValue];
			let tags = [];
			var opsList = _.forEach(this.state.ops, (o) => {
				if (selectCValue.indexOf(o.id) != -1)
				tags.push(<Tag key={o.id}>{o.name}</Tag>)
			});
			return tags;


		}


	},

	renderInputGroup () {
		// TODO: find better solution
		//   when importing the CreateForm using: import CreateForm from '../../../admin/client/App/shared/CreateForm';
		//   CreateForm was imported as a blank object. This stack overflow post suggested lazilly requiring it:
		// http://stackoverflow.com/questions/29807664/cyclic-dependency-returns-empty-object-in-react-native
		// TODO: Implement this somewhere higher in the app, it breaks the encapsulation of the RelationshipField component
		// const CreateForm = require('../../../admin/client/App/shared/CreateForm');
		// return (
		// 	<InputGroup>
		// 		<InputGroup.Section grow>
		// 			{this.renderSelect()}
		// 		</InputGroup.Section>
		// 		<InputGroup.Section>
		// 			<Button onClick={this.openCreate} type="success">+</Button>
		// 		</InputGroup.Section>
		// 		<CreateForm
		// 			list={listsByKey[this.props.refList.key]}
		// 			isOpen={this.state.createIsOpen}
		// 			onCreate={this.onCreate}
		// 			onCancel={this.closeCreate} />
		// 	</InputGroup>
		// );
		return null;
	},

	renderValue () {
		return this.renderSelect(true);
	},

	renderField () {
		const { inputProps, path } = this.props;
		return (<div>
			{this.props.createInline ? this.renderInputGroup() : this.renderSelect()}
			<Input
				name={this.getInputName(path)}
				type="hidden"
			/>
		</div>)

	},

});
