import React from 'react';
import { findDOMNode } from 'react-dom';
import {
	Form,
	Select,
	Input,
	Radio,
} from 'antdlib';
import _ from 'lodash';
import SegmentedControl from '../../components/SegmentedControl';
const INVERTED_OPTIONS = [
	{ label: '匹配', value: false },
	{ label: '不匹配', value: true },
];

const MODE_OPTIONS = [
	{ label: '包含', value: 'contains' },
	{ label: '等于', value: 'exactly' },
	{ label: '头部匹配', value: 'beginsWith' },
	{ label: '尾部匹配', value: 'endsWith' },
];

function getDefaultValue () {
	return {
		mode: MODE_OPTIONS[0].value,
		inverted: INVERTED_OPTIONS[0].value,
		value: '',
	};
}

var TextFilter = React.createClass({
	propTypes: {
		filter: React.PropTypes.shape({
			mode: React.PropTypes.oneOf(MODE_OPTIONS.map(i => i.value)),
			inverted: React.PropTypes.boolean,
			value: React.PropTypes.string,
		}),
	},
	statics: {
		getDefaultValue: getDefaultValue,
	},
	getDefaultProps () {
		return {
			filter: getDefaultValue(),
		};
	},
	componentWillMount(){
		console.log(this);
	},
	updateFilter (value) {
		this.props.onChange({ ...this.props.filter, ...value });
	},
	selectMode (mode) {
		this.updateFilter({ mode });
		findDOMNode(this.refs.focusTarget).focus();
	},
	toggleInverted (inverted) {
		this.updateFilter({ inverted });
		findDOMNode(this.refs.focusTarget).focus();
	},
	updateValue (e) {
		this.updateFilter({ value: e.target.value });
	},
	renderSegmentedControl(){
		 return _.map(INVERTED_OPTIONS, (o) => {
			return <Radio.Button key={o.label} value={o.value}>{o.label}</Radio.Button>
		 })
	},
	renderSelects(){
		const { field, filter } = this.props;
		const mode = MODE_OPTIONS.filter(i => i.value === filter.mode)[0];
		let options = _.map(MODE_OPTIONS, (o) => {
		   return <Select.Option key={o.value} value={o.value}>{o.label}</Select.Option>
		})

		return (<Select
			placeholder="选择匹配关系"
			onChange={this.selectMode.bind(this)}
			defaultValue={mode.value}
			getPopupContainer={() => findDOMNode(this.refs.main)}
		>
			{options}
		</Select>)
   },
	render () {
		const { field, filter } = this.props;
		const mode = MODE_OPTIONS.filter(i => i.value === filter.mode)[0];
		const placeholder = field.label + ' ' + mode.label.toLowerCase() + '...';
		// const formItemLayout = formLayout === 'horizontal' ? {
		// 	labelCol: { span: 4 },
		// 	wrapperCol: { span: 14 },
		//   } : null;
		return (
			<div ref="main">
				
				<Form.Item>
					<SegmentedControl options={INVERTED_OPTIONS} defaultValue={filter.inverted} onChange={this.toggleInverted} />
				</Form.Item>
				<Form.Item>
					{this.renderSelects()}
				</Form.Item>
				<Form.Item>
					<Input autoFocus ref="focusTarget" value={this.props.filter.value} onChange={this.updateValue} placeholder={placeholder} />
				</Form.Item>
			</div>
		);
	},
});

module.exports = TextFilter;
