import React from 'react';
import Field from '../Field';
import {AutoComplete, Button} from 'antdlib';
const AutoCompleteOption = AutoComplete.Option;

module.exports = Field.create({
	displayName: 'URLField',
	statics: {
		type: 'Url',
	},
	openValue () {
		var href = this.props.value;
		if (!href) return;
		if (!/^(mailto\:)|(\w+\:\/\/)/.test(href)) {
			href = 'http://' + href;
		}
		window.open(href);
	},
	getInitialState(){
		return  {
			autoCompleteResult: [],
		};
	},

	handleWebsiteChange(value){
		let autoCompleteResult;
		if (!value) {
			autoCompleteResult = [];
		} else {
			autoCompleteResult = ['.com', '.org', '.net'].map(domain => `${value}${domain}`);
		}
		this.setState({ autoCompleteResult });
	},
	renderLink () {
		if (!this.props.value) return null;

		return (
			<Button type="link" onClick={this.openValue} className="keystone-relational-button" title={'Open ' + this.props.value + ' in a new tab'}>
				<span className="octicon octicon-link" />
			</Button>
		);
	},
	renderField () {
		const {autoCompleteResult} = this.state;
		const websiteOptions = autoCompleteResult.map(website => (
			<AutoCompleteOption key={website}>{website}</AutoCompleteOption>
		));

		return (
			<AutoComplete
				size="large"
				dataSource={websiteOptions}
				name={this.getInputName(this.props.path)}
				onChange={this.handleWebsiteChange.bind(this)}
				placeholder="website url"
			/>
		);
	},
	// wrapField () {
	// 	return (
	// 		<div style={{ position: 'relative' }}>
	// 			{this.renderField()}
	// 			{this.renderLink()}
	// 		</div>
	// 	);
	// },
	renderValue () {
		return <Input disabled onClick={this.openValue}>{this.props.value}</Input>;
	},
});
