
// Require keystone
var keystone;

var _ = require("lodash");
var path = require("path");


var KAdm = function()
{
	this._options = {
		"kadmPath":"admin",
		'signin url':null,
		'signout url':null,
		'plugins':[],
		"adminModuleName":"keystone"
	};
	keystone = this.getAdminPlus();
}
_.extend(KAdm.prototype, require('./core/options')());
KAdm.prototype.getAdminPlus = () => require('./adminplus');
KAdm.prototype.getPath = () => {
	return __dirname;
}
KAdm.prototype.getKeystonePath = function()
{
	var str = path.join(path.dirname(require.resolve(this.get('adminModuleName'))), '..');
	return str;
}
KAdm.prototype.getKeystoneRelativePath = function(dir)
{
	var str = path.relative(dir, this.getKeystonePath());
	return str;
}
KAdm.prototype.init = function(keystoneParam)
{
	var initParam = {
		'name': 'KAdm',
		'brand': 'KStudio',
		'port':3001,
		'admin path': 'admincore',
		'less': 'public',
		'env' : "production",
		'headless' : true,
		//'static': 'public',
		'favicon': 'favicon.ico',
		'views': '/templates/views',
		'emails': 'templates/emails',
		'auto update': true,
		'session': true,
		'session store': 'mongo',
		'auth': true,
		'user model': 'User',
		'signin logo': '/images/logo.png',
		'uploads': 'uploads',
		'adminExtend':this,

		//wyi
		'wysiwyg option' : {
			menubar: false,
			height: 200,
			theme: 'modern',
			inline:true,
			skin: 'light',
			plugins: [
				'advlist autolink lists link image charmap print preview hr anchor pagebreak',
				'searchreplace wordcount visualblocks visualchars code fullscreen',
				'insertdatetime media nonbreaking save table contextmenu directionality',
				'emoticons template paste textcolor colorpicker textpattern imagetools codesample toc help'
			],
			toolbar1: 'code | undo redo | insert | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image',
			toolbar2: 'preview media | forecolor backcolor emoticons | codesample help',
			image_advtab: true,
		}
	};
	keystone.init(_.assignIn(initParam, keystoneParam));
	keystone.set('locals', {
		env: keystone.get('env'),
		utils: keystone.utils,
		editable: keystone.content.editable
	});

	// Load your project's Models
	keystone.import('models');

	//import all plugins Models
	let pluginList = keystone.get('plugins');
	_.forEach(pluginList, (o) => {
		let tmp;
		if (tmp = o.getModels()){
			let modelPath = path.relative(__dirname, tmp);
			keystone.import(modelPath);
		}
	});


	// Your cookie secret is used to secure session cookies. This environment
	// variable was added to your Heroku config for you if you used the "Deploy to
	// Heroku" button. The secret below will be used for development.
	// You may want to set it to something private and secure.

	if (!keystone.get('cookie secret')) {
		keystone.set('cookie secret', '----change-me-to-something-secret----');
	}

	// Setup common locals for your templates. The following are required for the
	// bundled templates and layouts. Any runtime locals (that should be set uniquely
	// for each request) should be added to ./routes/middleware.js

	keystone.set('locals', {
		env: keystone.get('env'),
		utils: keystone.utils,
		editable: keystone.content.editable
	});
	// Load your project's Routes
	keystone.set('routes', require('./admin/server/routes'));


	var email_hostname = process.env.EMAIL_HOSTNAME || 'localhost:3000';

}
KAdm.prototype.import = (dir) => {
	let kadmPath = path.join(process.cwd(), dir);
	let relatePath = path.relative(__dirname, kadmPath);
	keystone.import(relatePath);
}
KAdm.prototype.start = function()
{
	keystone.start({
		onStart:()=>{
			var rbac = require("./core/acl/RBACManager");
			this.rbac = rbac;
		}
	});

}
KAdm.prototype.package = require('./admin/client/prebuild/packages');
KAdm.prototype.Plugin = require('./plugin');
KAdm.prototype.List = require('./lib/list');
var adm = new KAdm();
//adm.Field = require(adm.getKeystonePath() + "/keystone/fields/types/Type"); //require('./fields/types/Type');
//adm.Field.Types = require(adm.getKeystonePath() + "/keystone/lib/fieldTypes") //require('./lib/fieldTypes');


module.exports = exports = adm;
adm.Field =  require('./fields/types/Type');
adm.Field.Types = require('./lib/fieldTypes');
adm.fieldTypes = {wysiwyg:true,};
_.forEach(adm.Field.Types, (value, key) => {
	adm.fieldTypes[key.toLowerCase()] = value.properName;
});
// adm.fieldType
