/**
 * Created by kinfen on 2017/11/17.
 */

import React from 'react';
import Picker from '../picker';
import reactCSS from 'reactcss';
import _ from 'lodash';
import {
	CustomPicker,
} from 'react-color';
import tinycolor from "tinycolor2";
import {
	Input,
	Col,
	Row,
} from "antdlib"
import {
	Alpha,
	EditableInput,
	Saturation,
	Checkboard,
	Hue,
	Swatch
} from 'react-color/lib/components/common';

const styleObj = reactCSS({
	'default': {
		swatchs: {
			position: "relative",
			margin: '8px 15px',
		},
		swatch: {
			width: '20px',
			height: '20px',
			float: 'left',
			borderRadius: '3px',
			margin: '0 6px 15px 0',
		},
		slideHeads:{
			margin:'15px',
		},
		slideGroup:{
			position: "relative",
			margin:'0 8px',
			height:30,
			display:'flex',
			flexDirection:'column',
			justifyContent:'space-between',

		},
		checkboard:{
			position: "relative",
			width:30,
			height:30,
			border:1,
			// borderRadius:'15px',
			overflow:'hidden',
		},
		colorboard:{
			absolute: '0px 0px 0px 0px',
			borderRadius: '0',
			boxShadow: 'inset 0 0 0 1px rgba(0,0,0,.1)',
			background: `#ff00ff`,
			zIndex: '2',
		},
		alpha: {
			position: "relative",
			height: 10,
			display:'flex',
			alignItem:"flex-end",

		},
		hue: {
			position: "relative",
			height: 10,
			display:'flex',
			alignItem:"flex-end",

		},
		saturation: {
			height: 100,
			position: "relative",
		},
		inputStyles: {
			input: {
				border: "none",
			},
			label: {
				fontSize: '12px',
				color: '#999',
			},
		}
	},
})


class ColorPicker extends Picker {

	constructor(props){
		super(props);

	}
	defaultProps = {
		colors: ['#FF6900', '#FCB900', '#7BDCB5', '#00D084', '#8ED1FC', '#0693E3',
			'#ABB8C3', '#EB144C', '#F78DA7', '#9900EF']
	}
	componentWillMount(){
		this.state = _.assign(this.state, this.defaultProps, this.toState(this.props.color || "#ffffff"));
	}
	toState(data, oldHue) {
		//计算各种颜色模式
		var color = data.hex ? tinycolor(data.hex) : tinycolor(data);
		var hsl = color.toHsl();
		var hsv = color.toHsv();
		var rgb = color.toRgb();
		var hex = color.toHex();
		if (hsl.s === 0) {
			hsl.h = oldHue || 0;
			hsv.h = oldHue || 0;
		}
		var transparent = hex === '000000' && rgb.a === 0;

		return {
			hsl: hsl,
			hex: transparent ? 'transparent' : '#' + hex,
			rgb: rgb,
			hsv: hsv,
			oldHue: data.h || oldHue || hsl.h,
			source: data.source
		};
	}
	handleFocusInput(){
		super.handleFocusInput();

	}
	componentDidUpdate(prevProps, prevState){

	}
	handleInputChange = (e) => {
		let colorHex = e.target.value
		let color = tinycolor(colorHex);
		if (color.isValid()){
			let cobj = this.toState(colorHex);
			this.setState(_.assign({},cobj, {input:colorHex}));
			this.props.onChange && this.props.onChange(cobj)
		}
		else {
			this.setState({
				input:colorHex,
			})
		}
	}
	handleChange = (color,event) => {
		let cobj = this.toState(color);
		this.setState(_.assign({},cobj, {input:cobj.hex}));
		this.props.onChange && this.props.onChange(cobj)
		this.refs.input.blur();
	}
	renderPickerContent(){

		const colorboard = _.assign({}, styleObj.colorboard, {backgroundColor:`rgba(${this.state.rgb.r}, ${this.state.rgb.g}, ${this.state.rgb.b}, ${this.state.rgb.a})`});
		return (
			<div style={{
				padding:0,
			}}>
				<Input ref="input" type="text" placeholder={this.state.hex} onChange={this.handleInputChange} value={this.state.input} />
				<div style={styleObj.saturation}>
					<Saturation
						{...this.state}
						{...{
							style:styleObj.saturation
						}}
						onChange={ this.handleChange.bind(this) }  />
				</div>
				<Row style={styleObj.slideHeads} justify="space-between" type="flex">
					<Col span={4} >
						<div style={styleObj.checkboard}>
							<Checkboard size={10} />
							<div style={colorboard}></div>
						</div>
					</Col>
					<Col span={20} >
						<div style={styleObj.slideGroup}>
							<div style={styleObj.hue}>
								<Hue
									{...this.state}
									onChange={ this.handleChange.bind(this) }
									direction={ 'horizontal' || 'vertical' } />
							</div>
							<div style={styleObj.alpha}>
								<Alpha
									{...this.state}
									onChange={ this.handleChange.bind(this) } />
							</div>
						</div>
					</Col>
				</Row>
				<div style={styleObj.swatchs}>
					{ _.map(this.state.colors, (c, i) => {
						return (
							<Swatch
								key={ i }
								color={ c }
								hex={ c }
								style={ styleObj.swatch }
								onClick={ this.handleChange.bind(this) }
								focusStyle={{
									boxShadow: `0 0 4px ${ c }`,
								}}
							/>
						)
					}) }
				</div>

			</div>
		)
	}

}

export default CustomPicker(ColorPicker)
